@@ -53,8 +53,10 @@ package Net::SAML;
 *zx_dup_simple_elem = *Net::SAMLc::zx_dup_simple_elem;
 *zx_ref_str = *Net::SAMLc::zx_ref_str;
 *zx_ref_len_str = *Net::SAMLc::zx_ref_len_str;
+*zx_new_len_str = *Net::SAMLc::zx_new_len_str;
 *zx_dup_str = *Net::SAMLc::zx_dup_str;
 *zx_dup_len_str = *Net::SAMLc::zx_dup_len_str;
+*zx_alloc_sprintf = *Net::SAMLc::zx_alloc_sprintf;
 *zx_strf = *Net::SAMLc::zx_strf;
 *zx_str_free = *Net::SAMLc::zx_str_free;
 *zx_str_to_c = *Net::SAMLc::zx_str_to_c;
@@ -65,6 +67,11 @@ package Net::SAML;
 *zx_zalloc = *Net::SAMLc::zx_zalloc;
 *zx_free = *Net::SAMLc::zx_free;
 *zx_dup_cstr = *Net::SAMLc::zx_dup_cstr;
+*zx_reset_ctx = *Net::SAMLc::zx_reset_ctx;
+*zx_init_ctx = *Net::SAMLc::zx_init_ctx;
+*zx_date_time_to_secs = *Net::SAMLc::zx_date_time_to_secs;
+*write2_or_append_lock_c_path = *Net::SAMLc::write2_or_append_lock_c_path;
+*zx_report_openssl_error = *Net::SAMLc::zx_report_openssl_error;
 *zx_fix_any_elem_dec = *Net::SAMLc::zx_fix_any_elem_dec;
 *zx_locate_ns_by_prefix = *Net::SAMLc::zx_locate_ns_by_prefix;
 *zx_is_ns_prefix = *Net::SAMLc::zx_is_ns_prefix;
@@ -72,6 +79,13 @@ package Net::SAML;
 *zx_prefix_seen_whine = *Net::SAMLc::zx_prefix_seen_whine;
 *zx_scan_xmlns = *Net::SAMLc::zx_scan_xmlns;
 *zx_pop_seen = *Net::SAMLc::zx_pop_seen;
+*zx_raw_cipher = *Net::SAMLc::zx_raw_cipher;
+*zx_rsa_pub_enc = *Net::SAMLc::zx_rsa_pub_enc;
+*zx_rsa_pub_dec = *Net::SAMLc::zx_rsa_pub_dec;
+*zx_rsa_priv_dec = *Net::SAMLc::zx_rsa_priv_dec;
+*zx_rsa_priv_enc = *Net::SAMLc::zx_rsa_priv_enc;
+*zx_get_rsa_pub_from_cert = *Net::SAMLc::zx_get_rsa_pub_from_cert;
+*zx_rand = *Net::SAMLc::zx_rand;
 *zx_tok_by_ns = *Net::SAMLc::zx_tok_by_ns;
 *zx_len_xmlns_if_not_seen = *Net::SAMLc::zx_len_xmlns_if_not_seen;
 *zx_enc_xmlns_if_not_seen = *Net::SAMLc::zx_enc_xmlns_if_not_seen;
@@ -86,19 +100,18 @@ package Net::SAML;
 *zx_attr_so_enc = *Net::SAMLc::zx_attr_so_enc;
 *zx_attr_wo_len = *Net::SAMLc::zx_attr_wo_len;
 *zx_attr_wo_enc = *Net::SAMLc::zx_attr_wo_enc;
-*zx_dup_attr = *Net::SAMLc::zx_dup_attr;
 *zx_free_attr = *Net::SAMLc::zx_free_attr;
-*zx_clone_attr = *Net::SAMLc::zx_clone_attr;
 *zx_free_elem_common = *Net::SAMLc::zx_free_elem_common;
-*zx_clone_elem_common = *Net::SAMLc::zx_clone_elem_common;
-*zx_dup_strs_common = *Net::SAMLc::zx_dup_strs_common;
-*zx_walk_so_unknown_attributes = *Net::SAMLc::zx_walk_so_unknown_attributes;
-*zx_walk_so_unknown_elems_and_content = *Net::SAMLc::zx_walk_so_unknown_elems_and_content;
-*zx_deep_clone_simple_elems = *Net::SAMLc::zx_deep_clone_simple_elems;
-*zx_walk_so_simple_elems = *Net::SAMLc::zx_walk_so_simple_elems;
 *zx_free_simple_elems = *Net::SAMLc::zx_free_simple_elems;
-*zx_dup_strs_simple_elems = *Net::SAMLc::zx_dup_strs_simple_elems;
 *zx_prepare_dec_ctx = *Net::SAMLc::zx_prepare_dec_ctx;
+*zx_scan_data = *Net::SAMLc::zx_scan_data;
+*zx_scan_pi_or_comment = *Net::SAMLc::zx_scan_pi_or_comment;
+*zx_dec_unknown_attr = *Net::SAMLc::zx_dec_unknown_attr;
+*zx_dec_attr_val = *Net::SAMLc::zx_dec_attr_val;
+*zx_xml_parse_err = *Net::SAMLc::zx_xml_parse_err;
+*zx_len_inc_ns = *Net::SAMLc::zx_len_inc_ns;
+*zx_add_inc_ns = *Net::SAMLc::zx_add_inc_ns;
+*zx_enc_inc_ns = *Net::SAMLc::zx_enc_inc_ns;
 *simple = *Net::SAMLc::zxid_simple;
 *idp_list = *Net::SAMLc::zxid_idp_list;
 *idp_select = *Net::SAMLc::zxid_idp_select;
@@ -115,14 +128,22 @@ package Net::SAML;
 *fed_mgmt_len = *Net::SAMLc::zxid_fed_mgmt_len;
 *idp_select_zxstr_cf = *Net::SAMLc::zxid_idp_select_zxstr_cf;
 *ses_to_ldif = *Net::SAMLc::zxid_ses_to_ldif;
+*simple_render_ses = *Net::SAMLc::zxid_simple_render_ses;
+*simple_ses_active_cf = *Net::SAMLc::zxid_simple_ses_active_cf;
+*simple_no_ses_cf = *Net::SAMLc::zxid_simple_no_ses_cf;
+*zxsig_sign = *Net::SAMLc::zxsig_sign;
 *zxsig_validate = *Net::SAMLc::zxsig_validate;
-*zx_report_openssl_error = *Net::SAMLc::zx_report_openssl_error;
 *zxsig_data_rsa_sha1 = *Net::SAMLc::zxsig_data_rsa_sha1;
 *zxsig_verify_data_rsa_sha1 = *Net::SAMLc::zxsig_verify_data_rsa_sha1;
+*zxenc_symkey_dec = *Net::SAMLc::zxenc_symkey_dec;
+*zxenc_privkey_dec = *Net::SAMLc::zxenc_privkey_dec;
+*zxenc_symkey_enc = *Net::SAMLc::zxenc_symkey_enc;
+*zxenc_pubkey_enc = *Net::SAMLc::zxenc_pubkey_enc;
 *zxlog_write_line = *Net::SAMLc::zxlog_write_line;
 *zxlog_path = *Net::SAMLc::zxlog_path;
 *zxlog_dup_check = *Net::SAMLc::zxlog_dup_check;
 *zxlog_blob = *Net::SAMLc::zxlog_blob;
+*zxlog = *Net::SAMLc::zxlog;
 *get_ent_from_file = *Net::SAMLc::zxid_get_ent_from_file;
 *get_ent_from_cache = *Net::SAMLc::zxid_get_ent_from_cache;
 *write_ent_to_cache = *Net::SAMLc::zxid_write_ent_to_cache;
@@ -134,9 +155,10 @@ package Net::SAML;
 *get_ent_by_succinct_id = *Net::SAMLc::zxid_get_ent_by_succinct_id;
 *get_ent_by_sha1_name = *Net::SAMLc::zxid_get_ent_by_sha1_name;
 *load_cot_cache = *Net::SAMLc::zxid_load_cot_cache;
+*key_info = *Net::SAMLc::zxid_key_info;
 *key_desc = *Net::SAMLc::zxid_key_desc;
 *slo_desc = *Net::SAMLc::zxid_slo_desc;
-*nireg_desc = *Net::SAMLc::zxid_nireg_desc;
+*mni_desc = *Net::SAMLc::zxid_mni_desc;
 *ac_desc = *Net::SAMLc::zxid_ac_desc;
 *sp_sso_desc = *Net::SAMLc::zxid_sp_sso_desc;
 *sp_meta = *Net::SAMLc::zxid_sp_meta;
@@ -158,11 +180,18 @@ package Net::SAML;
 *parse_conf = *Net::SAMLc::zxid_parse_conf;
 *parse_cgi = *Net::SAMLc::zxid_parse_cgi;
 *new_cgi = *Net::SAMLc::zxid_new_cgi;
+*get_sid_from_cookie = *Net::SAMLc::zxid_get_sid_from_cookie;
 *fetch_ses = *Net::SAMLc::zxid_fetch_ses;
 *get_ses = *Net::SAMLc::zxid_get_ses;
 *put_ses = *Net::SAMLc::zxid_put_ses;
 *del_ses = *Net::SAMLc::zxid_del_ses;
 *get_ses_sso_a7n = *Net::SAMLc::zxid_get_ses_sso_a7n;
+*get_ses_idp = *Net::SAMLc::zxid_get_ses_idp;
+*find_ses = *Net::SAMLc::zxid_find_ses;
+*put_user = *Net::SAMLc::zxid_put_user;
+*get_user_nameid = *Net::SAMLc::zxid_get_user_nameid;
+*user_change_nameid = *Net::SAMLc::zxid_user_change_nameid;
+*soap_call_raw = *Net::SAMLc::zxid_soap_call_raw;
 *lecp_check = *Net::SAMLc::zxid_lecp_check;
 *cdc_read = *Net::SAMLc::zxid_cdc_read;
 *cdc_check = *Net::SAMLc::zxid_cdc_check;
@@ -170,16 +199,18 @@ package Net::SAML;
 *soap_call_hdr_body = *Net::SAMLc::zxid_soap_call_hdr_body;
 *soap_call_body = *Net::SAMLc::zxid_soap_call_body;
 *soap_cgi_resp_body = *Net::SAMLc::zxid_soap_cgi_resp_body;
-*idp_loc_raw = *Net::SAMLc::zxid_idp_loc_raw;
-*idp_loc = *Net::SAMLc::zxid_idp_loc;
-*idp_soap = *Net::SAMLc::zxid_idp_soap;
 *redir_enc = *Net::SAMLc::zxid_saml2_redir_enc;
 *redir = *Net::SAMLc::zxid_saml2_redir;
 *redir_url = *Net::SAMLc::zxid_saml2_redir_url;
 *location = *Net::SAMLc::zxid_saml2_location;
-*resp_redir = *Net::SAMLc::zxid_saml2_resp_redir;
 *resp_location = *Net::SAMLc::zxid_saml2_resp_location;
+*resp_redir = *Net::SAMLc::zxid_saml2_resp_redir;
 *ok = *Net::SAMLc::zxid_saml_ok;
+*decrypt_nameid = *Net::SAMLc::zxid_decrypt_nameid;
+*decrypt_newnym = *Net::SAMLc::zxid_decrypt_newnym;
+*idp_loc_raw = *Net::SAMLc::zxid_idp_loc_raw;
+*idp_loc = *Net::SAMLc::zxid_idp_loc;
+*idp_soap = *Net::SAMLc::zxid_idp_soap;
 *pick_sso_profile = *Net::SAMLc::zxid_pick_sso_profile;
 *start_sso = *Net::SAMLc::zxid_start_sso;
 *start_sso_url = *Net::SAMLc::zxid_start_sso_url;
@@ -192,16 +223,19 @@ package Net::SAML;
 *date_time = *Net::SAMLc::zxid_date_time;
 *mk_id = *Net::SAMLc::zxid_mk_id;
 *issuer = *Net::SAMLc::zxid_issuer;
+*sigres_map = *Net::SAMLc::zxid_sigres_map;
 *sp_slo_soap = *Net::SAMLc::zxid_sp_slo_soap;
 *sp_slo_redir = *Net::SAMLc::zxid_sp_slo_redir;
 *sp_slo_location = *Net::SAMLc::zxid_sp_slo_location;
-*sp_nireg_soap = *Net::SAMLc::zxid_sp_nireg_soap;
-*sp_nireg_redir = *Net::SAMLc::zxid_sp_nireg_redir;
-*sp_nireg_location = *Net::SAMLc::zxid_sp_nireg_location;
 *sp_dispatch_location = *Net::SAMLc::zxid_sp_dispatch_location;
 *sp_dispatch = *Net::SAMLc::zxid_sp_dispatch;
 *sp_soap_dispatch = *Net::SAMLc::zxid_sp_soap_dispatch;
 *sp_soap_parse = *Net::SAMLc::zxid_sp_soap_parse;
+*sp_mni_soap = *Net::SAMLc::zxid_sp_mni_soap;
+*sp_mni_redir = *Net::SAMLc::zxid_sp_mni_redir;
+*sp_mni_location = *Net::SAMLc::zxid_sp_mni_location;
+*mni_do = *Net::SAMLc::zxid_mni_do;
+*mni_do_ss = *Net::SAMLc::zxid_mni_do_ss;
 *find_attribute = *Net::SAMLc::zxid_find_attribute;
 *mk_authn_req = *Net::SAMLc::zxid_mk_authn_req;
 *mk_art_deref = *Net::SAMLc::zxid_mk_art_deref;
@@ -209,8 +243,8 @@ package Net::SAML;
 *OK = *Net::SAMLc::zxid_OK;
 *mk_logout = *Net::SAMLc::zxid_mk_logout;
 *mk_logout_resp = *Net::SAMLc::zxid_mk_logout_resp;
-*mk_nireg = *Net::SAMLc::zxid_mk_nireg;
-*mk_nireg_resp = *Net::SAMLc::zxid_mk_nireg_resp;
+*mk_mni = *Net::SAMLc::zxid_mk_mni;
+*mk_mni_resp = *Net::SAMLc::zxid_mk_mni_resp;
 *mk_di_query = *Net::SAMLc::zxid_mk_di_query;
 *mk_addr = *Net::SAMLc::zxid_mk_addr;
 *mk_dap_select = *Net::SAMLc::zxid_mk_dap_select;
@@ -222,7 +256,6 @@ package Net::SAML;
 *mk_dap_query = *Net::SAMLc::zxid_mk_dap_query;
 *map_sec_mech = *Net::SAMLc::zxid_map_sec_mech;
 *wsc_call = *Net::SAMLc::zxid_wsc_call;
-*vzxid_new_envf = *Net::SAMLc::vzxid_new_envf;
 *new_envf = *Net::SAMLc::zxid_new_envf;
 *callf = *Net::SAMLc::zxid_callf;
 *get_epr = *Net::SAMLc::zxid_get_epr;
@@ -238,11 +271,12 @@ package Net::SAML;
 *zx_url_encode_raw = *Net::SAMLc::zx_url_encode_raw;
 *zx_url_encode = *Net::SAMLc::zx_url_encode;
 *read_all = *Net::SAMLc::read_all;
-*vname_from_path = *Net::SAMLc::vname_from_path;
 *name_from_path = *Net::SAMLc::name_from_path;
 *open_fd_from_path = *Net::SAMLc::open_fd_from_path;
 *read_all_fd = *Net::SAMLc::read_all_fd;
 *write_all_fd = *Net::SAMLc::write_all_fd;
+*write_all_path_fmt = *Net::SAMLc::write_all_path_fmt;
+*close_file = *Net::SAMLc::close_file;
 *curl_write_data = *Net::SAMLc::zxid_curl_write_data;
 *curl_read_data = *Net::SAMLc::zxid_curl_read_data;
 *version = *Net::SAMLc::zxid_version;
@@ -267,6 +301,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_seen_p_set = *Net::SAMLc::zx_ns_s_seen_p_set;
 *swig_seen_pop_get = *Net::SAMLc::zx_ns_s_seen_pop_get;
 *swig_seen_pop_set = *Net::SAMLc::zx_ns_s_seen_pop_set;
+*swig_inc_n_get = *Net::SAMLc::zx_ns_s_inc_n_get;
+*swig_inc_n_set = *Net::SAMLc::zx_ns_s_inc_n_set;
 *swig_prefix_len_get = *Net::SAMLc::zx_ns_s_prefix_len_get;
 *swig_prefix_len_set = *Net::SAMLc::zx_ns_s_prefix_len_set;
 *swig_prefix_get = *Net::SAMLc::zx_ns_s_prefix_get;
@@ -320,12 +356,24 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_lim_set = *Net::SAMLc::zx_ctx_lim_set;
 *swig_ns_tab_get = *Net::SAMLc::zx_ctx_ns_tab_get;
 *swig_ns_tab_set = *Net::SAMLc::zx_ctx_ns_tab_set;
+*swig_unknown_ns_get = *Net::SAMLc::zx_ctx_unknown_ns_get;
+*swig_unknown_ns_set = *Net::SAMLc::zx_ctx_unknown_ns_set;
 *swig_guard_seen_n_get = *Net::SAMLc::zx_ctx_guard_seen_n_get;
 *swig_guard_seen_n_set = *Net::SAMLc::zx_ctx_guard_seen_n_set;
 *swig_guard_seen_p_get = *Net::SAMLc::zx_ctx_guard_seen_p_get;
 *swig_guard_seen_p_set = *Net::SAMLc::zx_ctx_guard_seen_p_set;
 *swig_exclude_sig_get = *Net::SAMLc::zx_ctx_exclude_sig_get;
 *swig_exclude_sig_set = *Net::SAMLc::zx_ctx_exclude_sig_set;
+*swig_inc_ns_len_get = *Net::SAMLc::zx_ctx_inc_ns_len_get;
+*swig_inc_ns_len_set = *Net::SAMLc::zx_ctx_inc_ns_len_set;
+*swig_inc_ns_get = *Net::SAMLc::zx_ctx_inc_ns_get;
+*swig_inc_ns_set = *Net::SAMLc::zx_ctx_inc_ns_set;
+*swig_malloc_func_get = *Net::SAMLc::zx_ctx_malloc_func_get;
+*swig_malloc_func_set = *Net::SAMLc::zx_ctx_malloc_func_set;
+*swig_realloc_func_get = *Net::SAMLc::zx_ctx_realloc_func_get;
+*swig_realloc_func_set = *Net::SAMLc::zx_ctx_realloc_func_set;
+*swig_free_func_get = *Net::SAMLc::zx_ctx_free_func_get;
+*swig_free_func_set = *Net::SAMLc::zx_ctx_free_func_set;
 sub new {
     my $pkg = shift;
     my $self = Net::SAMLc::new_zx_ctx(@_);
@@ -637,12 +685,16 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_ctx_set = *Net::SAMLc::zxid_conf_ctx_set;
 *swig_cot_get = *Net::SAMLc::zxid_conf_cot_get;
 *swig_cot_set = *Net::SAMLc::zxid_conf_cot_set;
+*swig_path_supplied_get = *Net::SAMLc::zxid_conf_path_supplied_get;
+*swig_path_supplied_set = *Net::SAMLc::zxid_conf_path_supplied_set;
 *swig_path_len_get = *Net::SAMLc::zxid_conf_path_len_get;
 *swig_path_len_set = *Net::SAMLc::zxid_conf_path_len_set;
 *swig_path_get = *Net::SAMLc::zxid_conf_path_get;
 *swig_path_set = *Net::SAMLc::zxid_conf_path_set;
 *swig_url_get = *Net::SAMLc::zxid_conf_url_get;
 *swig_url_set = *Net::SAMLc::zxid_conf_url_set;
+*swig_non_standard_entityid_get = *Net::SAMLc::zxid_conf_non_standard_entityid_get;
+*swig_non_standard_entityid_set = *Net::SAMLc::zxid_conf_non_standard_entityid_set;
 *swig_cdc_url_get = *Net::SAMLc::zxid_conf_cdc_url_get;
 *swig_cdc_url_set = *Net::SAMLc::zxid_conf_cdc_url_set;
 *swig_cdc_choice_get = *Net::SAMLc::zxid_conf_cdc_choice_get;
@@ -655,16 +707,28 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_md_cache_first_set = *Net::SAMLc::zxid_conf_md_cache_first_set;
 *swig_md_cache_last_get = *Net::SAMLc::zxid_conf_md_cache_last_get;
 *swig_md_cache_last_set = *Net::SAMLc::zxid_conf_md_cache_last_set;
+*swig_user_local_get = *Net::SAMLc::zxid_conf_user_local_get;
+*swig_user_local_set = *Net::SAMLc::zxid_conf_user_local_set;
 *swig_authn_req_sign_get = *Net::SAMLc::zxid_conf_authn_req_sign_get;
 *swig_authn_req_sign_set = *Net::SAMLc::zxid_conf_authn_req_sign_set;
 *swig_want_sso_a7n_signed_get = *Net::SAMLc::zxid_conf_want_sso_a7n_signed_get;
 *swig_want_sso_a7n_signed_set = *Net::SAMLc::zxid_conf_want_sso_a7n_signed_set;
+*swig_sso_soap_sign_get = *Net::SAMLc::zxid_conf_sso_soap_sign_get;
+*swig_sso_soap_sign_set = *Net::SAMLc::zxid_conf_sso_soap_sign_set;
+*swig_sso_soap_resp_sign_get = *Net::SAMLc::zxid_conf_sso_soap_resp_sign_get;
+*swig_sso_soap_resp_sign_set = *Net::SAMLc::zxid_conf_sso_soap_resp_sign_set;
+*swig_nameid_enc_get = *Net::SAMLc::zxid_conf_nameid_enc_get;
+*swig_nameid_enc_set = *Net::SAMLc::zxid_conf_nameid_enc_set;
 *swig_affiliation_get = *Net::SAMLc::zxid_conf_affiliation_get;
 *swig_affiliation_set = *Net::SAMLc::zxid_conf_affiliation_set;
 *swig_nice_name_get = *Net::SAMLc::zxid_conf_nice_name_get;
 *swig_nice_name_set = *Net::SAMLc::zxid_conf_nice_name_set;
 *swig_ses_arch_dir_get = *Net::SAMLc::zxid_conf_ses_arch_dir_get;
 *swig_ses_arch_dir_set = *Net::SAMLc::zxid_conf_ses_arch_dir_set;
+*swig_ses_cookie_name_get = *Net::SAMLc::zxid_conf_ses_cookie_name_get;
+*swig_ses_cookie_name_set = *Net::SAMLc::zxid_conf_ses_cookie_name_set;
+*swig_ipport_get = *Net::SAMLc::zxid_conf_ipport_get;
+*swig_ipport_set = *Net::SAMLc::zxid_conf_ipport_set;
 *swig_log_err_get = *Net::SAMLc::zxid_conf_log_err_get;
 *swig_log_err_set = *Net::SAMLc::zxid_conf_log_err_set;
 *swig_log_act_get = *Net::SAMLc::zxid_conf_log_act_get;
@@ -681,6 +745,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_log_err_in_act_set = *Net::SAMLc::zxid_conf_log_err_in_act_set;
 *swig_log_sigfail_is_err_get = *Net::SAMLc::zxid_conf_log_sigfail_is_err_get;
 *swig_log_sigfail_is_err_set = *Net::SAMLc::zxid_conf_log_sigfail_is_err_set;
+*swig_log_level_get = *Net::SAMLc::zxid_conf_log_level_get;
+*swig_log_level_set = *Net::SAMLc::zxid_conf_log_level_set;
 *swig_sig_fatal_get = *Net::SAMLc::zxid_conf_sig_fatal_get;
 *swig_sig_fatal_set = *Net::SAMLc::zxid_conf_sig_fatal_set;
 *swig_nosig_fatal_get = *Net::SAMLc::zxid_conf_nosig_fatal_get;
@@ -697,8 +763,16 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_before_slop_set = *Net::SAMLc::zxid_conf_before_slop_set;
 *swig_after_slop_get = *Net::SAMLc::zxid_conf_after_slop_get;
 *swig_after_slop_set = *Net::SAMLc::zxid_conf_after_slop_set;
+*swig_timeskew_get = *Net::SAMLc::zxid_conf_timeskew_get;
+*swig_timeskew_set = *Net::SAMLc::zxid_conf_timeskew_set;
+*swig_redir_to_content_get = *Net::SAMLc::zxid_conf_redir_to_content_get;
+*swig_redir_to_content_set = *Net::SAMLc::zxid_conf_redir_to_content_set;
 *swig_max_soap_retry_get = *Net::SAMLc::zxid_conf_max_soap_retry_get;
 *swig_max_soap_retry_set = *Net::SAMLc::zxid_conf_max_soap_retry_set;
+*swig_defaultqs_get = *Net::SAMLc::zxid_conf_defaultqs_get;
+*swig_defaultqs_set = *Net::SAMLc::zxid_conf_defaultqs_set;
+*swig_mod_saml_attr_prefix_get = *Net::SAMLc::zxid_conf_mod_saml_attr_prefix_get;
+*swig_mod_saml_attr_prefix_set = *Net::SAMLc::zxid_conf_mod_saml_attr_prefix_set;
 *swig_idp_sel_start_get = *Net::SAMLc::zxid_conf_idp_sel_start_get;
 *swig_idp_sel_start_set = *Net::SAMLc::zxid_conf_idp_sel_start_set;
 *swig_idp_sel_new_idp_get = *Net::SAMLc::zxid_conf_idp_sel_new_idp_get;
@@ -723,6 +797,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_mgmt_footer_set = *Net::SAMLc::zxid_conf_mgmt_footer_set;
 *swig_mgmt_end_get = *Net::SAMLc::zxid_conf_mgmt_end_get;
 *swig_mgmt_end_set = *Net::SAMLc::zxid_conf_mgmt_end_set;
+*swig_dbg_get = *Net::SAMLc::zxid_conf_dbg_get;
+*swig_dbg_set = *Net::SAMLc::zxid_conf_dbg_set;
 sub new {
     my $pkg = shift;
     my $self = Net::SAMLc::new_zxid_conf(@_);
@@ -772,6 +848,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_ispassive_set = *Net::SAMLc::zxid_cgi_ispassive_set;
 *swig_force_authn_get = *Net::SAMLc::zxid_cgi_force_authn_get;
 *swig_force_authn_set = *Net::SAMLc::zxid_cgi_force_authn_set;
+*swig_enc_hint_get = *Net::SAMLc::zxid_cgi_enc_hint_get;
+*swig_enc_hint_set = *Net::SAMLc::zxid_cgi_enc_hint_set;
 *swig_sid_get = *Net::SAMLc::zxid_cgi_sid_get;
 *swig_sid_set = *Net::SAMLc::zxid_cgi_sid_set;
 *swig_nid_get = *Net::SAMLc::zxid_cgi_nid_get;
@@ -794,8 +872,16 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_matching_rule_set = *Net::SAMLc::zxid_cgi_matching_rule_set;
 *swig_authn_ctx_get = *Net::SAMLc::zxid_cgi_authn_ctx_get;
 *swig_authn_ctx_set = *Net::SAMLc::zxid_cgi_authn_ctx_set;
+*swig_pxy_count_get = *Net::SAMLc::zxid_cgi_pxy_count_get;
+*swig_pxy_count_set = *Net::SAMLc::zxid_cgi_pxy_count_set;
+*swig_get_complete_get = *Net::SAMLc::zxid_cgi_get_complete_get;
+*swig_get_complete_set = *Net::SAMLc::zxid_cgi_get_complete_set;
+*swig_idppxylist_get = *Net::SAMLc::zxid_cgi_idppxylist_get;
+*swig_idppxylist_set = *Net::SAMLc::zxid_cgi_idppxylist_set;
 *swig_rs_get = *Net::SAMLc::zxid_cgi_rs_get;
 *swig_rs_set = *Net::SAMLc::zxid_cgi_rs_set;
+*swig_newnym_get = *Net::SAMLc::zxid_cgi_newnym_get;
+*swig_newnym_set = *Net::SAMLc::zxid_cgi_newnym_set;
 *swig_saml_art_get = *Net::SAMLc::zxid_cgi_saml_art_get;
 *swig_saml_art_set = *Net::SAMLc::zxid_cgi_saml_art_set;
 *swig_saml_resp_get = *Net::SAMLc::zxid_cgi_saml_resp_get;
@@ -816,6 +902,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_msg_set = *Net::SAMLc::zxid_cgi_msg_set;
 *swig_dbg_get = *Net::SAMLc::zxid_cgi_dbg_get;
 *swig_dbg_set = *Net::SAMLc::zxid_cgi_dbg_set;
+*swig_idp_list_get = *Net::SAMLc::zxid_cgi_idp_list_get;
+*swig_idp_list_set = *Net::SAMLc::zxid_cgi_idp_list_set;
 sub new {
     my $pkg = shift;
     my $self = Net::SAMLc::new_zxid_cgi(@_);
@@ -859,6 +947,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_sid_set = *Net::SAMLc::zxid_ses_sid_set;
 *swig_nid_get = *Net::SAMLc::zxid_ses_nid_get;
 *swig_nid_set = *Net::SAMLc::zxid_ses_nid_set;
+*swig_sesix_get = *Net::SAMLc::zxid_ses_sesix_get;
+*swig_sesix_set = *Net::SAMLc::zxid_ses_sesix_set;
 *swig_nidfmt_get = *Net::SAMLc::zxid_ses_nidfmt_get;
 *swig_nidfmt_set = *Net::SAMLc::zxid_ses_nidfmt_set;
 *swig_sigres_get = *Net::SAMLc::zxid_ses_sigres_get;
@@ -869,6 +959,14 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_pad3_set = *Net::SAMLc::zxid_ses_pad3_set;
 *swig_sso_a7n_path_get = *Net::SAMLc::zxid_ses_sso_a7n_path_get;
 *swig_sso_a7n_path_set = *Net::SAMLc::zxid_ses_sso_a7n_path_set;
+*swig_setcookie_get = *Net::SAMLc::zxid_ses_setcookie_get;
+*swig_setcookie_set = *Net::SAMLc::zxid_ses_setcookie_set;
+*swig_cookie_get = *Net::SAMLc::zxid_ses_cookie_get;
+*swig_cookie_set = *Net::SAMLc::zxid_ses_cookie_set;
+*swig_rs_get = *Net::SAMLc::zxid_ses_rs_get;
+*swig_rs_set = *Net::SAMLc::zxid_ses_rs_set;
+*swig_nameid_get = *Net::SAMLc::zxid_ses_nameid_get;
+*swig_nameid_set = *Net::SAMLc::zxid_ses_nameid_set;
 *swig_a7n_get = *Net::SAMLc::zxid_ses_a7n_get;
 *swig_a7n_set = *Net::SAMLc::zxid_ses_a7n_set;
 *swig_a7n11_get = *Net::SAMLc::zxid_ses_a7n11_get;
@@ -918,6 +1016,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 %ITERATORS = ();
 *swig_n_get = *Net::SAMLc::zxid_entity_n_get;
 *swig_n_set = *Net::SAMLc::zxid_entity_n_set;
+*swig_n_cdc_get = *Net::SAMLc::zxid_entity_n_cdc_get;
+*swig_n_cdc_set = *Net::SAMLc::zxid_entity_n_cdc_set;
 *swig_eid_len_get = *Net::SAMLc::zxid_entity_eid_len_get;
 *swig_eid_len_set = *Net::SAMLc::zxid_entity_eid_len_set;
 *swig_eid_get = *Net::SAMLc::zxid_entity_eid_get;
@@ -973,6 +1073,10 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_sref_set = *Net::SAMLc::zxsig_ref_sref_set;
 *swig_blob_get = *Net::SAMLc::zxsig_ref_blob_get;
 *swig_blob_set = *Net::SAMLc::zxsig_ref_blob_set;
+*swig_id_get = *Net::SAMLc::zxsig_ref_id_get;
+*swig_id_set = *Net::SAMLc::zxsig_ref_id_set;
+*swig_canon_get = *Net::SAMLc::zxsig_ref_canon_get;
+*swig_canon_set = *Net::SAMLc::zxsig_ref_canon_set;
 sub new {
     my $pkg = shift;
     my $self = Net::SAMLc::new_zxsig_ref(@_);
@@ -1064,10 +1168,28 @@ package Net::SAML;
 *ZX_TOK_XMLNS = *Net::SAMLc::ZX_TOK_XMLNS;
 *ZX_TOK_DATA = *Net::SAMLc::ZX_TOK_DATA;
 *ZX_TOK_NOT_FOUND = *Net::SAMLc::ZX_TOK_NOT_FOUND;
+*SIG_ALGO_RSA_SHA1 = *Net::SAMLc::SIG_ALGO_RSA_SHA1;
+*SIG_ALGO_DSA_SHA1 = *Net::SAMLc::SIG_ALGO_DSA_SHA1;
+*SIG_ALGO = *Net::SAMLc::SIG_ALGO;
+*DIGEST_ALGO = *Net::SAMLc::DIGEST_ALGO;
+*CANON_ALGO = *Net::SAMLc::CANON_ALGO;
+*ENVELOPED_ALGO = *Net::SAMLc::ENVELOPED_ALGO;
+*ENC_ALGO_TRIPLEDES_CBC = *Net::SAMLc::ENC_ALGO_TRIPLEDES_CBC;
+*ENC_ALGO_AES128_CBC = *Net::SAMLc::ENC_ALGO_AES128_CBC;
+*ENC_ALGO_AES192_CBC = *Net::SAMLc::ENC_ALGO_AES192_CBC;
+*ENC_ALGO_AES256_CBC = *Net::SAMLc::ENC_ALGO_AES256_CBC;
+*ENC_ALGO = *Net::SAMLc::ENC_ALGO;
+*ENC_KEYTRAN_RSA_1_5 = *Net::SAMLc::ENC_KEYTRAN_RSA_1_5;
+*ENC_KEYTRAN_RSA_OAEP = *Net::SAMLc::ENC_KEYTRAN_RSA_OAEP;
+*ENC_KEYTRAN_ALGO = *Net::SAMLc::ENC_KEYTRAN_ALGO;
+*ENC_ENCKEY_METH = *Net::SAMLc::ENC_ENCKEY_METH;
+*ENC_TYPE_ELEMENT = *Net::SAMLc::ENC_TYPE_ELEMENT;
+*ENC_TYPE_CONTENT = *Net::SAMLc::ENC_TYPE_CONTENT;
 *CONF_MAGIC = *Net::SAMLc::ZXID_CONF_MAGIC;
 *CGI_MAGIC = *Net::SAMLc::ZXID_CGI_MAGIC;
 *SES_MAGIC = *Net::SAMLc::ZXID_SES_MAGIC;
 *SES_DIR = *Net::SAMLc::ZXID_SES_DIR;
+*USER_DIR = *Net::SAMLc::ZXID_USER_DIR;
 *AUTO_EXIT = *Net::SAMLc::ZXID_AUTO_EXIT;
 *AUTO_REDIR = *Net::SAMLc::ZXID_AUTO_REDIR;
 *AUTO_SOAPC = *Net::SAMLc::ZXID_AUTO_SOAPC;
@@ -1096,6 +1218,10 @@ package Net::SAML;
 *ZXLOG_ISSUE_DIR = *Net::SAMLc::ZXLOG_ISSUE_DIR;
 *ZXLOG_A7N_KIND = *Net::SAMLc::ZXLOG_A7N_KIND;
 *ZXLOG_MSG_KIND = *Net::SAMLc::ZXLOG_MSG_KIND;
+*COT_DIR = *Net::SAMLc::ZXID_COT_DIR;
+*MAX_MD = *Net::SAMLc::ZXID_MAX_MD;
+*MAX_SOAP = *Net::SAMLc::ZXID_MAX_SOAP;
+*MAX_EID = *Net::SAMLc::ZXID_MAX_EID;
 *DAP_SCOPE_BASE = *Net::SAMLc::ZXID_DAP_SCOPE_BASE;
 *DAP_SCOPE_SINGLE = *Net::SAMLc::ZXID_DAP_SCOPE_SINGLE;
 *DAP_SCOPE_SUBTREE = *Net::SAMLc::ZXID_DAP_SCOPE_SUBTREE;
@@ -1104,14 +1230,16 @@ package Net::SAML;
 *CDC_CHOICE_ALWAYS_ONLY = *Net::SAMLc::ZXID_CDC_CHOICE_ALWAYS_ONLY;
 *CDC_CHOICE_UI_PREF = *Net::SAMLc::ZXID_CDC_CHOICE_UI_PREF;
 *CDC_CHOICE_UI_NOPREF = *Net::SAMLc::ZXID_CDC_CHOICE_UI_NOPREF;
+*CDC_CHOICE_UI_ONLY_CDC = *Net::SAMLc::ZXID_CDC_CHOICE_UI_ONLY_CDC;
 *ART = *Net::SAMLc::ZXID_SAML2_ART;
 *POST = *Net::SAMLc::ZXID_SAML2_POST;
 *PAOS = *Net::SAMLc::ZXID_SAML2_PAOS;
 *SOAP = *Net::SAMLc::ZXID_SAML2_SOAP;
+*POST_SIMPLE_SIGN = *Net::SAMLc::ZXID_SAML2_POST_SIMPLE_SIGN;
 *REDIR = *Net::SAMLc::ZXID_SAML2_REDIR;
 *URI = *Net::SAMLc::ZXID_SAML2_URI;
 *SLO_SVC = *Net::SAMLc::ZXID_SLO_SVC;
-*NIREG_SVC = *Net::SAMLc::ZXID_NIREG_SVC;
+*MNI_SVC = *Net::SAMLc::ZXID_MNI_SVC;
 *SEC_MECH_NULL = *Net::SAMLc::ZXID_SEC_MECH_NULL;
 *SEC_MECH_BEARER = *Net::SAMLc::ZXID_SEC_MECH_BEARER;
 *SEC_MECH_SAML = *Net::SAMLc::ZXID_SEC_MECH_SAML;
@@ -1179,6 +1307,7 @@ package Net::SAML;
 *REDIR = *Net::SAMLc::SAML2_REDIR;
 *ART = *Net::SAMLc::SAML2_ART;
 *POST = *Net::SAMLc::SAML2_POST;
+*POST_SIMPLE_SIGN = *Net::SAMLc::SAML2_POST_SIMPLE_SIGN;
 *URI = *Net::SAMLc::SAML2_URI;
 *AP_BASIC = *Net::SAMLc::SAML2_AP_BASIC;
 *AP_X500 = *Net::SAMLc::SAML2_AP_X500;
