@@ -1,22 +1,20 @@
 /* zxidmeta.c  -  Handwritten functions for metadata parsing and generation as well as CoT handling
- * Copyright (c) 2006-2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Copyright (c) 2006-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxidmeta.c,v 1.31 2007/03/28 20:31:54 sampo Exp $
+ * $Id: zxidmeta.c,v 1.45 2008-05-30 17:39:11 lbernardo Exp $
  *
- * 12.8.2006, created --Sampo
- *
- * See also: http://hoohoo.ncsa.uiuc.edu/cgi/interface.html (CGI specification)
- *           http://curl.haxx.se/libcurl/
+ * 12.8.2006,  created --Sampo
+ * 12.10.2007, mild refactoring to process keys for xenc as well. --Sampo
+ * 13.12.2007, fixed missing KeyDescriptor/@use as seen in CA IdP metadata --Sampo
+ * 14.4.2008,  added SimpleSign --Sampo
  */
 
-//#include <pthread.h>
 #include <signal.h>
 #include <fcntl.h>
-//#include <netdb.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,21 +23,18 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-//#include <sys/wait.h>
 
 #ifdef USE_OPENSSL
 #include <openssl/sha.h>
 #include <openssl/x509.h>
 #include <openssl/rsa.h>
 #endif
-#ifdef USE_CURL
-#include <curl/curl.h>
-#endif
 
 #include "errmac.h"
 #include "saml2.h"
 #include "zxid.h"
 #include "zxidconf.h"
+#include "platform.h"
 #include "c/zx-const.h"
 #include "c/zx-ns.h"
 #include "c/zx-data.h"
@@ -53,102 +48,9 @@
 
 /* ============== CoT and Metadata of Others ============== */
 
-#define ZXID_COT_DIR "cot/"
-#define ZXID_MAX_MD (64*1024)
-
-/* Called by: */
-size_t zxid_curl_write_data(void *buffer, size_t size, size_t nmemb, void *userp)
-{
-  int len = size*nmemb;
-#if 1
-  struct zxid_curl_ctx* rc = (struct zxid_curl_ctx*)userp;
-  if (rc->p + len > rc->lim) {
-    ERR("Metadata is too long. Wont fit in %d byte buffer.", ZXID_MAX_MD);
-    return 0;
-  }
-  memcpy(rc->p, buffer, len);
-  rc->p += len;
-  D("RECV(%.*s) %d chars", len, (char*)buffer, len);
-#else
-  int fd = (int)userp;
-  write_all_fd(fd, buffer, len);
-#endif
-  return len;
-}
-
-/* Called by: */
-size_t zxid_curl_read_data(void *buffer, size_t size, size_t nmemb, void *userp)
-{
-  int len = size*nmemb;
-  struct zxid_curl_ctx* wc = (struct zxid_curl_ctx*)userp;
-  if (len > (wc->lim - wc->p))
-    len = wc->lim - wc->p;
-  memcpy(buffer, wc->p, len);
-  wc->p += len;
-  D("SEND(%.*s) %d chars", len, (char*)buffer, len);
-  return len;
-}
-
-/* Called by:  opt, zxid_get_meta_ss */
-struct zxid_entity* zxid_get_meta(struct zxid_conf* cf, char* url)
-{
-  struct zxid_entity* ent;
-  CURLcode res;
-#ifdef USE_CURL
-#if 1
-  struct zxid_curl_ctx rc;
-  char* md_buf = ZX_ALLOC(cf->ctx, ZXID_MAX_MD+1);
-  rc.p = md_buf;
-  rc.lim = md_buf + ZXID_MAX_MD;
-  curl_easy_setopt(cf->curl, CURLOPT_WRITEDATA, &rc);
-#else
-  /* TEST CODE (usually disabled) */
-  int fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, "get_meta TEST", "%s" ZXID_COT_DIR "test", cf->path);
-  if (fd == -1) {
-    perror("open meta to write");
-    ERR("Failed to open file(%s) for writing metadata", url);
-    return 0;
-  }
-  curl_easy_setopt(cf->curl, CURLOPT_WRITEDATA, fd);
-#endif
-  curl_easy_setopt(cf->curl, CURLOPT_WRITEFUNCTION, zxid_curl_write_data);
-  curl_easy_setopt(cf->curl, CURLOPT_NOPROGRESS, 1);
-  curl_easy_setopt(cf->curl, CURLOPT_FOLLOWLOCATION, 1);
-  curl_easy_setopt(cf->curl, CURLOPT_MAXREDIRS, 110);
-  curl_easy_setopt(cf->curl, CURLOPT_SSL_VERIFYPEER, 0);
-  curl_easy_setopt(cf->curl, CURLOPT_SSL_VERIFYHOST, 0);
-  curl_easy_setopt(cf->curl, CURLOPT_URL, url);
-
-  D("get_meta: url(%s)", url);
-  res = curl_easy_perform(cf->curl);
-  /*close(fd);*/
-  rc.lim = rc.p;
-  rc.p[1] = 0;
-  rc.p = md_buf;
-  if (rc.lim - rc.p < 300) {
-    ERR("Metadata response too short (%d chars, min 300 required). url(%s) CURLcode(%d) CURLerr(%s) buf(%.*s)", rc.lim-md_buf, url, res, curl_easy_strerror(res), rc.lim-md_buf, md_buf);
-    ZX_FREE(cf->ctx, md_buf);
-    return 0;
-  }
-
-  ent = zxid_parse_meta(cf, &rc.p, rc.lim);
-  if (!ent) {
-    ERR("Failed to parse metadata response url(%s) CURLcode(%d) CURLerr(%s) buf(%.*s)", url, res, curl_easy_strerror(res), rc.lim-md_buf, md_buf);
-    ZX_FREE(cf->ctx, md_buf);
-    return 0;
-  }
-  return ent;
-#else
-  ERR("This copy of zxid was compiled to NOT use libcurl. As such metadata fetching is not supported. Add -DUSE_CURL and recompile. %d", 0);
-  return 0;
-#endif
-}
-
-/* Called by:  zxid_get_ent_ss */
-struct zxid_entity* zxid_get_meta_ss(struct zxid_conf* cf, struct zx_str* url)
-{
-  return zxid_get_meta(cf, zx_str_to_c(cf->ctx, url));
-}
+/* Process certificates (public keys) from a metadata for entity.
+ * Since one entity can be both IdP and SP, this function may
+ * be called twice per entity, with different kd argument. */
 
 /* Called by:  zxid_parse_meta x2 */
 static void zxid_process_keys(struct zxid_conf* cf, struct zxid_entity* ent,
@@ -158,7 +60,7 @@ static void zxid_process_keys(struct zxid_conf* cf, struct zxid_entity* ent,
   char* pp;
   char* p;
   char* e;
-  X509* x = 0;  /* Forces d2i_X509() to alloc the memory. */
+  X509* x;
 
   for (; kd; kd = (struct zx_md_KeyDescriptor_s*)kd->gg.g.n) {
     p = kd->KeyInfo->X509Data->X509Certificate->content->s;
@@ -166,11 +68,18 @@ static void zxid_process_keys(struct zxid_conf* cf, struct zxid_entity* ent,
     e = p + len;
     pp = ZX_ALLOC(cf->ctx, SIMPLE_BASE64_PESSIMISTIC_DECODE_LEN(e-p));
     e = unbase64_raw(p, e, pp, zx_std_index_64);
+    x = 0;  /* Forces d2i_X509() to alloc the memory. */
     if (!d2i_X509(&x, (const unsigned char**)&pp, e-pp) || !x) {
       ERR("DER decoding of X509 certificate for %s failed. use(%.*s)", logkey, kd->use->len, kd->use->s);
       D("Extracted %s base64 form of cert(%.*s)", logkey, len, p);
       return;
     }
+    if (!kd->use) {
+      ent->sign_cert = x;
+      ent->enc_cert = x;
+      D("KeyDescriptor is missing use attribute. Assume this certificate can be used for both signing and encryption. %d", 0);
+      return;
+    }
     if (!memcmp("signing", kd->use->s, kd->use->len)) {
       ent->sign_cert = x;
       DD("Extracted %s sign cert(%.*s)", logkey, len, p);
@@ -191,8 +100,11 @@ struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
   struct zx_md_EntityDescriptor_s* ed;
   struct zx_root_s* r;
 
+  DD("!!!!!!!!!!!!!!lim(%s)", lim);
   zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, *md, lim);
+  DD("???????????????lim(%s)", lim);
   r = zx_DEC_root(cf->ctx, 0, 5);
+  DD("@@@@@@@@@@@@@@@@@lim(%s)", lim);
   *md = cf->ctx->p;
   if (!r)
     return 0;
@@ -224,6 +136,7 @@ struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
   
  bad_md:
   ERR("Bad metadata. EntityDescriptor could not be found or was corrupt. MD(%.*s) %d chars parsed.", lim-cf->ctx->base, cf->ctx->base, *md - cf->ctx->base);
+  zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "B", "BADMD", 0, "chars_parsed(%d)", *md - cf->ctx->base);
   zx_FREE_root(cf->ctx, r, 0);
   return 0;
 }
@@ -232,8 +145,8 @@ struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
 int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
 {
   struct zx_str* ss;
-  int fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, "write_ent_to_cache", "%s" ZXID_COT_DIR "%s", cf->path, ent->sha1_name);
-  if (fd == -1) {
+  fdtype fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, "write_ent_to_cache", "%s" ZXID_COT_DIR "%s", cf->path, ent->sha1_name);
+  if (fd == BADFD) {
     perror("open metadata for writing metadata to cache");
     ERR("Failed to open file for writing: sha1_name(%s) to metadata cache", ent->sha1_name);
     return 0;
@@ -244,20 +157,22 @@ int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
     return 0;
   write_all_fd(fd, ss->s, ss->len);
   zx_str_free(cf->ctx, ss);
-  close(fd);
+  close_file(fd, __FUNCTION__);
   return 1;
 }
 
 /* Called by:  main, zxid_get_ent_by_sha1_name, zxid_get_ent_from_cache */
 struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name)
 {
-  int fd,n,got;
+  int n,got;
+  fdtype fd;
   char* md_buf;
   char* p;
   struct zxid_entity* ent;
   
+  D("sha1_name(%s)", sha1_name);
   fd = open_fd_from_path(O_RDONLY, 0, "get_ent_from_file", "%s" ZXID_COT_DIR "%s", cf->path, sha1_name);
-  if (fd == -1) {
+  if (fd == BADFD) {
     perror("open metadata to read");
     D("No metadata file found for sha1_name(%s)", sha1_name);
     return 0;
@@ -265,23 +180,26 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
   
   md_buf = ZX_ALLOC(cf->ctx, ZXID_MAX_MD+1);
   n = read_all_fd(fd, md_buf, ZXID_MAX_MD, &got);
+  D("==========sha1_name(%s)", sha1_name);
   if (n == -1) {
     perror("read metadata");
-    ERR("Failed to read metadata for sha1_name(%s)", sha1_name);
-    close(fd);
+    D("Failed to read metadata for sha1_name(%s)", sha1_name);
+    close_file(fd, __FUNCTION__);
     return 0;
   }
-  close(fd);
+  close_file(fd, __FUNCTION__);
   
   p = md_buf;
   ent = zxid_parse_meta(cf, &p, md_buf+got);
+  D("++++++++++++sha1_name(%s)", sha1_name);
   if (!ent) {
     ZX_FREE(cf->ctx, md_buf);
+    ERR("***** Parsing metadata failed for sha1_name(%s)", sha1_name);
     return 0;
   }
   ent->n = cf->cot;
   cf->cot = ent;
-  D("GOT sha1_name(%s) eid(%.*s)", sha1_name, ent->eid_len, ent->eid);
+  D("GOT META sha1_name(%s) eid(%.*s)", sha1_name, ent->eid_len, ent->eid);
   return ent;
 }
 
@@ -291,8 +209,10 @@ struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str*
   struct zxid_entity* ent;
   char sha1_name[28];
   for (ent = cf->cot; ent; ent = ent->n)  /* Check in memory cache. */
-    if (eid->len == ent->eid_len && !memcmp(eid->s, ent->eid, eid->len))
+    if (eid->len == ent->eid_len && !memcmp(eid->s, ent->eid, eid->len)) {
+      D("GOT FROM MEM eid(%.*s)", ent->eid_len, ent->eid);
       return ent;
+    }
   sha1_safe_base64(sha1_name, eid->len, eid->s);
   sha1_name[27] = 0;
   return zxid_get_ent_from_file(cf, sha1_name);
@@ -303,7 +223,7 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
 {
   struct zxid_entity* ent;
   
-  D("get_ent: eid(%.*s) path(%.*s) cf->magic=%x\n", eid->len, eid->s, cf->path_len, cf->path, cf->magic);
+  D("eid(%.*s) path(%.*s) cf->magic=%x, md_cache_first(%d), cot(%x)", eid->len, eid->s, cf->path_len, cf->path, cf->magic, cf->md_cache_first, cf->cot);
   if (cf->md_cache_first) {
     ent = zxid_get_ent_from_cache(cf, eid);
     if (ent)
@@ -317,7 +237,7 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
       cf->cot = ent;
       
       if (cf->md_populate_cache)
-	zxid_write_ent_to_cache(cf, ent);
+        zxid_write_ent_to_cache(cf, ent);
       return ent;
     }
   }
@@ -327,7 +247,8 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
     if (ent)
       return ent;
   }
-  D("get_ent: eid(%.*s) NOT FOUND\n", eid->len, eid->s);
+  D("eid(%.*s) NOT FOUND\n", eid->len, eid->s);
+  zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "B", "NOMD", 0, "eid(%.*s)", eid->len, eid->s);
   return 0;
 }
 
@@ -339,6 +260,7 @@ struct zxid_entity* zxid_get_ent(struct zxid_conf* cf, char* eid)
     return 0;
   ss.s = eid;
   ss.len = strlen(eid);
+  D("eid: (%s)\n", eid);
   return zxid_get_ent_ss(cf, &ss);
 }
 
@@ -349,16 +271,17 @@ struct zxid_entity* zxid_get_ent_by_sha1_name(struct zxid_conf* cf, char* sha1_n
   for (ent = cf->cot; ent; ent = ent->n)  /* Check in-memory cache. */
     if (!strcmp(sha1_name, ent->sha1_name))
       return ent;
-  D("Trying sha1_name(%s)", sha1_name);
-  return zxid_get_ent_from_file(cf, sha1_name);
+  ent = zxid_get_ent_from_file(cf, sha1_name);
+  if (!ent)
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "B", "NOMD", 0, "sha1_name(%s)", sha1_name);
+  return ent;
 }
 
 /* Called by:  zxid_sp_deref_art */
 struct zxid_entity* zxid_get_ent_by_succinct_id(struct zxid_conf* cf, char* raw_succinct_id)
 {
-  struct zxid_entity* ent;
   char sha1_name[28];
-  char* p = base64_fancy_raw(raw_succinct_id, 20, sha1_name, safe_basis_64, 1<<31, 0, 0, '.');
+  base64_fancy_raw(raw_succinct_id, 20, sha1_name, safe_basis_64, 1<<31, 0, 0, '.');
   sha1_name[27] = 0;
   return zxid_get_ent_by_sha1_name(cf, sha1_name);
 }
@@ -375,9 +298,8 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
   struct dirent* de;
   DIR* dir;
   char buf[4096];
-  int base_len, dir_len, len, fd;
   if (cf->path_len + sizeof(ZXID_COT_DIR) > sizeof(buf)) {
-    ERR("Too long path(%.*s) for config dir. Has %d chars. Max allowed %d. (config problem)",
+   ERR("Too long path(%.*s) for config dir. Has %d chars. Max allowed %d. (config problem)",
 	cf->path_len, cf->path, cf->path_len, sizeof(buf) - sizeof(ZXID_COT_DIR));
     return 0;
   }
@@ -392,9 +314,10 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
   }
   
   while (de = readdir(dir))
-    if (de->d_name[0] != '.')
+    if (de->d_name[0] != '.' && de->d_name[strlen(de->d_name)-1] != '~')
       zxid_get_ent_by_sha1_name(cf, de->d_name);
   
+  D("HERE %p", cf);
   closedir(dir);
   return cf->cot;
 }
@@ -445,18 +368,15 @@ struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X50
 }
 #endif
 
-/* Called by:  zxid_sp_sso_desc x2 */
-struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
+struct zx_ds_KeyInfo_s* zxid_key_info(struct zxid_conf* cf, X509* x)
 {
   int len;
   char* dd;
   char* d;
   char* pp;
   char* p;
-  struct zx_md_KeyDescriptor_s* kd = zx_NEW_md_KeyDescriptor(cf->ctx);
-  kd->use = zx_ref_str(cf->ctx, use);
-  kd->KeyInfo = zx_NEW_ds_KeyInfo(cf->ctx);
-  kd->KeyInfo->X509Data = zx_NEW_ds_X509Data(cf->ctx);
+  struct zx_ds_KeyInfo_s* ki = zx_NEW_ds_KeyInfo(cf->ctx);
+  ki->X509Data = zx_NEW_ds_X509Data(cf->ctx);
 
 #ifdef USE_OPENSSL
   /* Build PEM encoding (which is base64 of the DER encoding + header and footer) */
@@ -470,11 +390,20 @@ struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X50
     pp = p = ZX_ALLOC(cf->ctx, (len+4) * 4 / 3 + (len/64) + 6);    
     p = base64_fancy_raw(dd, len, p, std_basis_64, 64, 1, "\n", '=');
     *p = 0;
-    kd->KeyInfo->X509Data->X509Certificate = zx_ref_len_simple_elem(cf->ctx, p-pp, pp);
+    ki->X509Data->X509Certificate = zx_ref_len_simple_elem(cf->ctx, p-pp, pp);
   }
 #else
   ERR("This copy of zxid was compiled to NOT use OpenSSL. Generating KeyInfo is not supported. Add -DUSE_OPENSSL and recompile. %d", 0);
 #endif
+  return ki;
+}
+
+/* Called by:  zxid_sp_sso_desc x2 */
+struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
+{
+  struct zx_md_KeyDescriptor_s* kd = zx_NEW_md_KeyDescriptor(cf->ctx);
+  kd->use = zx_ref_str(cf->ctx, use);
+  kd->KeyInfo = zxid_key_info(cf, x);
   return kd;
 }
 
@@ -491,8 +420,8 @@ struct zx_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
 }
 
 /* Called by:  zxid_sp_sso_desc x2 */
-struct zx_md_ManageNameIDService_s* zxid_nireg_desc(struct zxid_conf* cf,
-						      char* binding, char* loc, char* resp_loc)
+struct zx_md_ManageNameIDService_s* zxid_mni_desc(struct zxid_conf* cf,
+						  char* binding, char* loc, char* resp_loc)
 {
   struct zx_md_ManageNameIDService_s* d = zx_NEW_md_ManageNameIDService(cf->ctx);
   d->Binding = zx_ref_str(cf->ctx, binding);
@@ -504,7 +433,7 @@ struct zx_md_ManageNameIDService_s* zxid_nireg_desc(struct zxid_conf* cf,
 
 /* Called by:  zxid_sp_sso_desc x4 */
 struct zx_md_AssertionConsumerService_s* zxid_ac_desc(struct zxid_conf* cf,
-							char* binding, char* loc, char* ix)
+						      char* binding, char* loc, char* ix)
 {
   struct zx_md_AssertionConsumerService_s* d = zx_NEW_md_AssertionConsumerService(cf->ctx);
   d->Binding = zx_ref_str(cf->ctx, binding);
@@ -540,12 +469,12 @@ struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
     zk->gg.g.n = &sp_ssod->KeyDescriptor->gg.g;
     sp_ssod->KeyDescriptor = zk;
     
-    zk = zxid_key_desc(cf, "signing", cf->enc_cert);
+    zk = zxid_key_desc(cf, "signing", cf->sign_cert);
     zk->gg.g.n = &sp_ssod->KeyDescriptor->gg.g;
     sp_ssod->KeyDescriptor = zk;
   }
 
-  z2 = zxid_slo_desc(cf, SAML2_REDIR, "?o=K", "?o=K");
+  z2 = zxid_slo_desc(cf, SAML2_REDIR, "?o=Q", "?o=Q");
   z2->gg.g.n = &sp_ssod->SingleLogoutService->gg.g;
   sp_ssod->SingleLogoutService = z2;
 
@@ -553,11 +482,11 @@ struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
   z2->gg.g.n = &sp_ssod->SingleLogoutService->gg.g;
   sp_ssod->SingleLogoutService = z2;
 
-  z3 = zxid_nireg_desc(cf, SAML2_REDIR, "?o=I", "?o=I");
+  z3 = zxid_mni_desc(cf, SAML2_REDIR, "?o=Q", "?o=Q");
   z3->gg.g.n = &sp_ssod->ManageNameIDService->gg.g;
   sp_ssod->ManageNameIDService = z3;
 
-  z3 = zxid_nireg_desc(cf, SAML2_SOAP, "?o=S", 0);
+  z3 = zxid_mni_desc(cf, SAML2_SOAP, "?o=S", 0);
   z3->gg.g.n = &sp_ssod->ManageNameIDService->gg.g;
   sp_ssod->ManageNameIDService = z3;
 
@@ -572,6 +501,10 @@ struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
   /* N.B. The index values should not be changed. They are used in
    * AuthnReq to choose profile using AssertionConsumerServiceIndex */
 
+  za = zxid_ac_desc(cf, SAML2_POST_SIMPLE_SIGN, "?o=P", "5");
+  za->gg.g.n = &sp_ssod->AssertionConsumerService->gg.g;
+  sp_ssod->AssertionConsumerService = za;
+
   za = zxid_ac_desc(cf, SAML2_PAOS, "?o=P", "4");
   za->gg.g.n = &sp_ssod->AssertionConsumerService->gg.g;
   sp_ssod->AssertionConsumerService = za;
@@ -594,8 +527,13 @@ struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
 /* Called by:  main, zxid_idp_select, zxid_mk_ecp_Request_hdr, zxid_my_issuer, zxid_sp_meta, zxid_sp_sso_finalize */
 struct zx_str* zxid_my_entity_id(struct zxid_conf* cf)
 {
-  D("my_entity_id url(%s)", cf->url);
-  return zx_strf(cf->ctx, "%s?o=B", cf->url);
+  if (cf->non_standard_entityid) {
+    D("my_entity_id non_standard_entytid(%s)", cf->non_standard_entityid);
+    return zx_strf(cf->ctx, "%s", cf->non_standard_entityid);
+  } else {
+    D("my_entity_id url(%s)", cf->url);
+    return zx_strf(cf->ctx, "%s?o=B", cf->url);
+  }
 }
 
 /* Called by: */
@@ -615,7 +553,7 @@ struct zx_sa_Issuer_s* zxid_issuer(struct zxid_conf* cf, struct zx_str* nameid,
   return is;
 }
 
-/* Called by:  zxid_mk_art_deref x2, zxid_mk_authn_req, zxid_mk_ecp_Request_hdr, zxid_mk_logout x2, zxid_mk_logout_resp x2, zxid_mk_nireg, zxid_mk_nireg_resp */
+/* Called by:  zxid_mk_art_deref x2, zxid_mk_authn_req, zxid_mk_ecp_Request_hdr, zxid_mk_logout x2, zxid_mk_logout_resp x2, zxid_mk_mni, zxid_mk_mni_resp */
 struct zx_sa_Issuer_s* zxid_my_issuer(struct zxid_conf* cf) {
   return zxid_issuer(cf, zxid_my_entity_id(cf), cf->affiliation);
 }
@@ -629,6 +567,8 @@ struct zx_str* zxid_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
   ed->entityID = zxid_my_entity_id(cf);
   ed->SPSSODescriptor = zxid_sp_sso_desc(cf);
   
+  if (cf->log_level>0)
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "MYMD", 0, 0);
   return zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ed);
 }
 
@@ -638,7 +578,8 @@ int zxid_send_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
   struct zx_str* ss = zxid_sp_meta(cf, cgi);
   if (!ss)
     return 0;
-  write_all_fd(1, ss->s, ss->len);
+  //write_all_fd(1, ss->s, ss->len);
+  write_all_fd(stdout, ss->s, ss->len);
   zx_str_free(cf->ctx, ss);
   return 0;
 }
