@@ -1,18 +1,20 @@
 /* zxidsimp.c  -  Handwritten zxid_simple() API
- * Copyright (c) 2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Copyright (c) 2007-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxidsimp.c,v 1.19 2007-06-19 15:17:03 sampo Exp $
+ * $Id: zxidsimp.c,v 1.41 2008-05-30 17:39:11 lbernardo Exp $
  *
  * 17.1.2007, created --Sampo
  * 2.2.2007, improved the LDIF return --Sampo
+ * 9.3.2008, refactored the logged in and need login cases to subroutines --Sampo
  *
  * Login button abbreviations
  * A2 = SAML 2.0 Artifact Profile
  * P2 = SAML 2.0 POST Profile
+ * '' = SAML 2.0 POST Simple Sign
  * A12 = Liberty ID-FF 1.2 Artifact Profile
  * P12 = Liberty ID-FF 1.2 POST Profile
  * A1 = Bare SAML 1.x Artifact Profile
@@ -23,27 +25,48 @@
 
 #include <memory.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "errmac.h"
+#include "zx.h"
 #include "zxid.h"
 #include "zxidconf.h"
 
 /* Called by:  zxid_fed_mgmt_len, zxid_idp_list_len, zxid_new_conf_to_cf, zxid_simple_len */
 int zxid_conf_to_cf_len(struct zxid_conf* cf, int conf_len, char* conf)
 {
+#if 1
+  if (!cf->ctx) {
+    cf->ctx = zx_init_ctx();
+    if (!cf->ctx) {
+      ERR("Failed to alloc zx_ctx %d",0);
+      exit(2);
+    }
+  }
+  zxid_init_conf(cf, ZXID_PATH);
+#ifdef USE_CURL
+  cf->curl = curl_easy_init();
+  if (!cf->curl) {
+    ERR("Failed to initialize libcurl %d",0);
+    exit(2);
+  }
+#endif
+#else
   zxid_init_conf_ctx(cf, ZXID_PATH /* N.B. Often this is overridden. */);
+#endif
 #if defined(ZXID_CONF_FILE) || defined(ZXID_CONF_FLAG)
   {
     char* buf;
     char* cc;
     int len, clen = conf_len == -1 && conf ? strlen(conf) : conf_len;
     if (conf && conf_len) {
+      /* Copy the conf string because wqe are going to modify it in place. */
       cc = ZX_ALLOC(cf->ctx, clen+1);
       memcpy(cc, conf, clen);
       cc[clen] = 0;
-      buf = ZX_ALLOC(cf->ctx, ZXID_MAX_CONF);
       zxid_parse_conf_raw(cf, clen, cc);
     }
+    buf = ZX_ALLOC(cf->ctx, ZXID_MAX_CONF);
     len = read_all(ZXID_MAX_CONF-1, buf, "-conf_to_cf", "%szxid.conf", cf->path);
     if (len > 0) {
       buf[len] = 0;
@@ -64,20 +87,28 @@ struct zxid_conf* zxid_new_conf_to_cf(char* conf)
     ERR("out-of-memory %d", sizeof(struct zxid_conf));
     exit(1);
   }
+  cf = memset(cf, 0, sizeof(struct zxid_conf));
   zxid_conf_to_cf_len(cf, -1, conf);
   return cf;
 }
 
 /* ------------ zxid_fed_mgmt() ------------ */
+/* Either outputs the management screen to stdout or returns string of HTML (at specified
+ * automation level). If res_len is supplied, the string length is returned in res_len.
+ * Otherwise you can just run strlen() on return value. */
 
 /* Called by:  zxid_fed_mgmt_len */
 char* zxid_fed_mgmt_cf(struct zxid_conf* cf, int* res_len, int sid_len, char* sid, int auto_flags)
 {
-  char* s;
+  char* res;
   struct zx_str* ss;
+  struct zx_str* ss2;
   int slen = sid_len == -1 && sid ? strlen(sid) : sid_len;
   zxid_set_opt(cf, 1, auto_flags & ZXID_AUTO_DEBUG?1:0);
 
+  if (cf->log_level>1)
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "MGMT", 0, "sid(%.*s)", sid_len, sid);
+  
   if ((auto_flags & ZXID_AUTO_FORMT) && (auto_flags & ZXID_AUTO_FORMF))
     ss = zx_strf(cf->ctx,
 		 "%s"
@@ -88,12 +119,12 @@ char* zxid_fed_mgmt_cf(struct zxid_conf* cf, int* res_len, int sid_len, char* si
 #endif
 		 "<input type=hidden name=s value=\"%.*s\">"
 		 "%s%s"
-		 "</form>%s%s%s",
+		 "</form>%s%s%s%s",
 		 cf->mgmt_start,
 		 cf->url,
 		 slen, sid,
 		 cf->mgmt_logout, cf->mgmt_defed,
-		 cf->mgmt_footer, zxid_version_str(), cf->mgmt_end);
+		 cf->mgmt_footer, zxid_version_str(), STRNULLCHK(cf->dbg), cf->mgmt_end);
   else if (auto_flags & ZXID_AUTO_FORMT)
     ss = zx_strf(cf->ctx,
 #ifdef ZXID_USE_POST
@@ -129,11 +160,35 @@ char* zxid_fed_mgmt_cf(struct zxid_conf* cf, int* res_len, int sid_len, char* si
   if (cgi->dbg) printf("<p><form><textarea cols=100 row=10>%s</textarea></form>\n", cgi->dbg);
 #endif
 
-  s = ss->s;
+  if (auto_flags & ZXID_AUTO_MGMTC && auto_flags & ZXID_AUTO_MGMTH) {  /* Both H&C: CGI */
+    printf("Content-Type: text/html" CRLF "Content-Length: %d" CRLF2 "%.*s",
+	   ss->len, ss->len, ss->s);
+    zx_str_free(cf->ctx, ss);
+    return 0;
+  }
+
+  if (auto_flags & (ZXID_AUTO_MGMTC | ZXID_AUTO_MGMTH)) {
+    if (auto_flags & ZXID_AUTO_MGMTH) {  /* H only: return both H and C */
+      D("With headers 0x%x", auto_flags);
+      ss2 = zx_strf(cf->ctx, "Content-Type: text/html" CRLF "Content-Length: %d" CRLF2 "%.*s",
+		    ss->len, ss->len, ss->s);
+      zx_str_free(cf->ctx, ss);
+    } else {
+      D("No headers 0x%x", auto_flags);
+      ss2 = ss;       /* C only */
+    }
+    res = ss2->s;
+    DD("res(%s)", res);
+    if (res_len)
+      *res_len = ss2->len;
+    ZX_FREE(cf->ctx, ss2);
+    return res;
+  }
+  D("m(%.*s)", ss->len, ss->s);
+  zx_str_free(cf->ctx, ss);
   if (res_len)
-    *res_len = ss->len;
-  ZX_FREE(cf->ctx, ss);
-  return s;
+    *res_len = 1;
+  return zx_dup_cstr(cf->ctx, "m");   /* Neither H nor C */
 }
 
 /* Called by:  zxid_fed_mgmt */
@@ -152,13 +207,15 @@ char* zxid_fed_mgmt(char* conf, char* sid, int auto_flags)
 
 /* ------------ zxid_idp_list() ------------ */
 
-/* Called by:  zxid_idp_list_len, zxid_idp_select */
-char* zxid_idp_list_cf(struct zxid_conf* cf, int* res_len, int auto_flags)
+char* zxid_idp_list_cf_cgi(struct zxid_conf* cf, struct zxid_cgi* cgi, int* res_len, int auto_flags)
 {
+  int i;
   char* s;
+  char mark[32];
   struct zx_str* ss;
   struct zx_str* dd;
   struct zxid_entity* idp;
+  struct zxid_entity* idp_cdc;
   zxid_set_opt(cf, 1, auto_flags & ZXID_AUTO_DEBUG?1:0);
   idp = zxid_load_cot_cache(cf);
   if (!idp) {
@@ -172,19 +229,63 @@ char* zxid_idp_list_cf(struct zxid_conf* cf, int* res_len, int auto_flags)
     ss = zx_dup_str(cf->ctx, "<h3>Login Using Known IdP</h3>\n");
   else
     ss = zx_dup_str(cf->ctx, "");
+
+#define ZXID_USE_POPUP
+#ifdef ZXID_USE_POPUP
+  dd = zx_strf(cf->ctx, "%.*s<select name=d>\n", ss->len, ss->s);
+  zx_str_free(cf->ctx, ss);
+  ss = dd;
+#endif
+
   D("Starting IdP processing... %p", idp);
   for (; idp; idp = idp->n) {
     if (!idp->ed->IDPSSODescriptor)
       continue;
+    
+    mark[0] = 0;
+    if (cgi) {    /* Was IdP recommended in IdP list supplied via CDC? See zxid_cdc_check() */
+      for (idp_cdc = cgi->idp_list, i=1; idp_cdc && idp_cdc != idp; idp_cdc = idp_cdc->n_cdc, ++i);
+      if (cf->cdc_choice == ZXID_CDC_CHOICE_UI_ONLY_CDC && cgi->idp_list && !idp_cdc)
+	continue;
+      if (idp_cdc) {
+	snprintf(mark, sizeof(mark), " CDC %d", i);
+	mark[sizeof(mark)-1] = 0;
+      }
+    }
+
+#ifdef ZXID_USE_POPUP
     dd = zx_strf(cf->ctx, "%.*s"
-		 "<input type=submit name=\"l1%.*s\" value=\" Login to %.*s (A2) \">\n"
-		 "<input type=submit name=\"l2%.*s\" value=\" Login to %.*s (P2) \"><br>\n",
+		 "<option value=\"%.*s\"> %.*s %s\n",
 		 ss->len, ss->s,
 		 idp->eid_len, idp->eid, idp->eid_len, idp->eid,
-		 idp->eid_len, idp->eid, idp->eid_len, idp->eid);
+		 mark);
+#else
+    dd = zx_strf(cf->ctx, "%.*s"
+		 "<input type=submit name=\"l0%.*s\" value=\" Login to %.*s \">\n"
+		 "<input type=submit name=\"l1%.*s\" value=\" Login to %.*s (A2) \">\n"
+		 "<input type=submit name=\"l2%.*s\" value=\" Login to %.*s (P2) \">\n"
+		 "<input type=submit name=\"l5%.*s\" value=\" Login to %.*s (S2) \">%s<br>\n",
+		 ss->len, ss->s,
+		 idp->eid_len, idp->eid, idp->eid_len, idp->eid,
+		 idp->eid_len, idp->eid, idp->eid_len, idp->eid,
+		 idp->eid_len, idp->eid, idp->eid_len, idp->eid,
+		 idp->eid_len, idp->eid, idp->eid_len, idp->eid,
+		 mark);
+#endif
     zx_str_free(cf->ctx, ss);
     ss = dd;
   }
+#ifdef ZXID_USE_POPUP
+  dd = zx_strf(cf->ctx, "%.*s</select>"
+	       "<input type=submit name=\"l0\" value=\" Login \">\n"
+	       "<input type=submit name=\"l1\" value=\" Login (A2) \">\n"
+	       "<input type=submit name=\"l2\" value=\" Login (P2) \">\n"
+	       "<input type=submit name=\"l5\" value=\" Login (S2) \"><br>\n",
+	       ss->len, ss->s);
+  zx_str_free(cf->ctx, ss);
+  ss = dd;
+#endif
+
   s = ss->s;
   D("IdP list(%s)", s);
   if (res_len)
@@ -193,6 +294,12 @@ char* zxid_idp_list_cf(struct zxid_conf* cf, int* res_len, int auto_flags)
   return s;
 }
 
+/* Called by:  zxid_idp_list_len, zxid_idp_select */
+char* zxid_idp_list_cf(struct zxid_conf* cf, int* res_len, int auto_flags)
+{
+  return zxid_idp_list_cf_cgi(cf, 0, res_len, auto_flags);
+}
+
 /* Called by:  zxid_idp_list */
 char* zxid_idp_list_len(int conf_len, char* conf, int* res_len, int auto_flags)
 {
@@ -208,18 +315,21 @@ char* zxid_idp_list(char* conf, int auto_flags)
 }
 
 /* Called by:  zxid_simple_cf */
-struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
+struct zx_str* zxid_idp_select_zxstr_cf_cgi(struct zxid_conf* cf, struct zxid_cgi* cgi, int auto_flags)
 {
   struct zx_str* eid=0;
   struct zx_str* ss;
   char* idp_list;
 
-  DD("HERE %p %p", cf, cf->idp_sel_our_eid);
+  D("HERE %p %p", cf, cf->idp_sel_our_eid);
   if (cf->idp_sel_our_eid && cf->idp_sel_our_eid[0])
     eid = zxid_my_entity_id(cf);
-  idp_list = zxid_idp_list_cf(cf, 0, auto_flags);
-  DD("idp_list(%s)", idp_list?idp_list:"?");
+  D("HERE %p", eid);
+  idp_list = zxid_idp_list_cf_cgi(cf, cgi, 0, auto_flags);
+  if (cf->log_level>1)
+    zxlog(cf, 0,0,0,0,0,0,0, "N", "W", "IDPSEL", 0, 0);
   if ((auto_flags & ZXID_AUTO_FORMT) && (auto_flags & ZXID_AUTO_FORMF)) {
+    D("HERE %p", idp_list);
     ss = zx_strf(cf->ctx,
 		 "%s"
 #ifdef ZXID_USE_POST
@@ -231,6 +341,7 @@ struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
 		 "%s<a href=\"%.*s\">%.*s</a><br>"
 		 "%s"    /* IdP List */
 		 "%s%s"
+		 "<input type=hidden name=fr value=\"%s\">"
 		 "</form>%s%s%s",
 		 cf->idp_sel_start,
 		 cf->url,
@@ -238,7 +349,9 @@ struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
 		 cf->idp_sel_our_eid, eid?eid->len:0, eid?eid->s:"", eid?eid->len:0, eid?eid->s:"",
 		 idp_list,
 		 cf->idp_sel_tech_user, cf->idp_sel_tech_site,
+		 (cgi&&cgi->rs)?cgi->rs:"",
 		 cf->idp_sel_footer, zxid_version_str(), cf->idp_sel_end);
+    D("HERE(%d) ss(%.*s)", ss->len, ss->len, ss->s);
   } else if (auto_flags & ZXID_AUTO_FORMT) {
     ss = zx_strf(cf->ctx,
 #ifdef ZXID_USE_POST
@@ -250,22 +363,26 @@ struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
 		 "%s<a href=\"%.*s\">%.*s</a><br>"
 		 "%s"    /* IdP List */
 		 "%s%s"
+		 "<input type=hidden name=fr value=\"%s\">"
 		 "</form>",
 		 cf->url,
 		 cf->idp_sel_new_idp,
 		 cf->idp_sel_our_eid, eid?eid->len:0, eid?eid->s:"", eid?eid->len:0, eid?eid->s:"",
 		 idp_list,
-		 cf->idp_sel_tech_user, cf->idp_sel_tech_site);
+		 cf->idp_sel_tech_user, cf->idp_sel_tech_site,
+		 (cgi&&cgi->rs)?cgi->rs:"");
   } else if (auto_flags & ZXID_AUTO_FORMF) {
     ss = zx_strf(cf->ctx,
 		 "%s"
 		 "%s<a href=\"%.*s\">%.*s</a><br>"
 		 "%s"    /* IdP List */
-		 "%s%s",
+		 "%s%s"
+		 "<input type=hidden name=fr value=\"%s\">",
 		 cf->idp_sel_new_idp,
 		 cf->idp_sel_our_eid, eid?eid->len:0, eid?eid->s:"", eid?eid->len:0, eid?eid->s:"",
 		 idp_list,
-		 cf->idp_sel_tech_user, cf->idp_sel_tech_site);
+		 cf->idp_sel_tech_user, cf->idp_sel_tech_site,
+		 (cgi&&cgi->rs)?cgi->rs:"");
   } else
     ss = zx_dup_str(cf->ctx, "");
 #if 0
@@ -280,6 +397,11 @@ struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
   return ss;
 }
 
+struct zx_str* zxid_idp_select_zxstr_cf(struct zxid_conf* cf, int auto_flags)
+{
+  return zxid_idp_select_zxstr_cf_cgi(cf, 0, auto_flags);
+}
+
 char* zxid_idp_select_cf(struct zxid_conf* cf, int* res_len, int auto_flags)
 {
   char* s;
@@ -316,6 +438,8 @@ struct zx_str* zxid_ses_to_ldif(struct zxid_conf* cf, struct zxid_ses* ses)
   struct zx_sa_AttributeValue_s* av;
   struct zx_sa_Attribute_s* at;
   struct zx_sa_AttributeStatement_s* as;
+  struct zx_str* issuer;
+  struct zx_str* accr;
   struct zx_str* ss;
   zxid_get_ses_sso_a7n(cf, ses);
   
@@ -356,6 +480,10 @@ struct zx_str* zxid_ses_to_ldif(struct zxid_conf* cf, struct zxid_ses* ses)
   
   /* Format some pseudo attributes that describe the SSO */
   
+  issuer = ses->a7n&&ses->a7n->Issuer&&ses->a7n->Issuer->gg.content?ses->a7n->Issuer->gg.content:0;
+
+  accr = ses->a7n&&ses->a7n->AuthnStatement&&ses->a7n->AuthnStatement->AuthnContext&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content?ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content:0;
+
   ss = zx_strf(cf->ctx,
 "dn: idpnid=%s,affid=%.*s\n"
 "objectclass: zxidsession\n"
@@ -363,22 +491,19 @@ struct zx_str* zxid_ses_to_ldif(struct zxid_conf* cf, struct zxid_ses* ses)
 "idpnid: %s\n"
 "authnctxlevel: %.*s\n"
 "sesid: %s\n"
-"%.*s\n"
+"setcookie: %s\n"
+"cookie: %s\n"
+"%.*s"
+"rs: %s\n\n"
 	       , ses->nid?ses->nid:"-",
-
-	       ses->a7n&&ses->a7n->Issuer&&ses->a7n->Issuer->gg.content&&ses->a7n->Issuer->gg.content->len?ses->a7n->Issuer->gg.content->len:1,
-	       ses->a7n&&ses->a7n->Issuer&&ses->a7n->Issuer->gg.content&&ses->a7n->Issuer->gg.content->len?ses->a7n->Issuer->gg.content->s:"-",
-
-	       ses->a7n&&ses->a7n->Issuer&&ses->a7n->Issuer->gg.content&&ses->a7n->Issuer->gg.content->len?ses->a7n->Issuer->gg.content->len:1,
-	       ses->a7n&&ses->a7n->Issuer&&ses->a7n->Issuer->gg.content&&ses->a7n->Issuer->gg.content->len?ses->a7n->Issuer->gg.content->s:"-",
-
+	       issuer&&issuer->len?issuer->len:1, issuer&&issuer->len?issuer->s:"-",
+	       issuer&&issuer->len?issuer->len:1, issuer&&issuer->len?issuer->s:"-",
 	       ses->nid?ses->nid:"-",
-
-	       ses->a7n&&ses->a7n->AuthnStatement&&ses->a7n->AuthnStatement->AuthnContext&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content->len?ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content->len:1,
-	       ses->a7n&&ses->a7n->AuthnStatement&&ses->a7n->AuthnStatement->AuthnContext&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content->len?ses->a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content->s:"-",
-	       
+	       accr&&accr->len?accr->len:1, accr&&accr->len?accr->s:"-",
 	       ses->sid?ses->sid:"-",
-	       len, q);
+	       ses->setcookie?ses->setcookie:"-",
+	       ses->cookie?ses->cookie:"-",
+	       len, q, ses->rs?ses->rs:"-");
   
   ZX_FREE(cf->ctx, q);
   return ss;
@@ -386,166 +511,239 @@ struct zx_str* zxid_ses_to_ldif(struct zxid_conf* cf, struct zxid_ses* ses)
 
 /* ------------ zxid_simple() ------------ */
 
-/* Simple handler that assumes the configuration has already been read in.
- * The memory for result is grabbed from ZX_ALLOC(), usually malloc(3)
- * and is "given" away to the caller, i.e. caller must free it. The
- * return value is LDIF of attributes in success case.
- * res_len, if non-null, will receive the length of the response. */
-
-/* Called by:  zxid_simple_len */
-char* zxid_simple_cf(struct zxid_conf* cf, int qs_len, char* qs, int* res_len, int auto_flags)
+char* zxid_simple_render_ses(struct zxid_conf* cf, struct zxid_ses* ses, int* res_len, int auto_flags)
 {
-  int got, ret;
-  char* cont_len;
-  char* buf = 0;
+  char* res;
+  struct zx_str* ss;
+  DD("render_ses %d", 0);
+  ss = zxid_ses_to_ldif(cf, ses);
+  D("LDIF(%.*s)", ss?ss->len:1, ss?ss->s:"-");
+  if (cf->log_level > 0)
+    zxlog(cf, 0,0,0,0,0,0, ses->nameid?ses->nameid->gg.content:0, "N", "K", "SHOWPC", ses->sid, 0);
+  res = ss->s;
+  if (res_len)
+    *res_len = ss->len;
+  ZX_FREE(cf->ctx, ss);
+  return res;
+}
+
+static char* zxid_simple_show_login(struct zxid_conf* cf, struct zxid_cgi* cgi, int* res_len, int auto_flags)
+{
+  char* res;
+  struct zx_str* ss;
+  struct zx_str* ss2;
+  D("cf=%p cgi=%p", cf, cgi);
+  ss = zxid_idp_select_zxstr_cf_cgi(cf, cgi, auto_flags);
+  DD("idp_select: ret(%s)", ss?ss->len:1, ss?ss->s:"?");
+  if (auto_flags & ZXID_AUTO_LOGINC && auto_flags & ZXID_AUTO_LOGINH) {  /* Both H&C: CGI */
+    D("CGI %x", auto_flags);
+    printf("Content-Type: text/html" CRLF "Content-Length: %d" CRLF2 "%.*s",
+	   ss->len, ss->len, ss->s);
+    if (auto_flags & ZXID_AUTO_EXIT)
+      exit(0);
+    zx_str_free(cf->ctx, ss);
+    if (res_len)
+      *res_len = 1;
+    return zx_dup_cstr(cf->ctx, "n");
+  }
+  
+  if (auto_flags & (ZXID_AUTO_LOGINC | ZXID_AUTO_LOGINH)) {
+    if (auto_flags & ZXID_AUTO_LOGINH) {  /* H only: return both H and C */
+      D("With headers %x", auto_flags);
+      ss2 = zx_strf(cf->ctx, "Content-Type: text/html" CRLF "Content-Length: %d" CRLF2 "%.*s",
+		    ss->len, ss->len, ss->s);
+      zx_str_free(cf->ctx, ss);
+    } else {
+      D("No headers %x", auto_flags);
+      ss2 = ss;       /* C only */
+    }
+    res = ss2->s;
+    DD("res(%s)", res);
+    if (res_len)
+      *res_len = ss2->len;
+    ZX_FREE(cf->ctx, ss2);
+    return res;
+  }
+  D("e(%.*s)", ss->len, ss->s);
+  zx_str_free(cf->ctx, ss);
+  if (res_len)
+    *res_len = 1;
+  return zx_dup_cstr(cf->ctx, "e");   /* Neither H nor C */
+}
+
+static char* zxid_simple_show_meta(struct zxid_conf* cf, struct zxid_cgi* cgi, int* res_len, int auto_flags)
+{
+  char* res;
+  struct zx_str* meta;
+  struct zx_str* ss;
+
+  meta = zxid_sp_meta(cf, cgi);
+  if (auto_flags & ZXID_AUTO_METAC && auto_flags & ZXID_AUTO_METAH) {
+    printf("Content-Type: text/xml" CRLF "Content-Length: %d" CRLF2 "%.*s",
+	   meta->len, meta->len, meta->s);
+    if (auto_flags & ZXID_AUTO_EXIT)
+      exit(0);
+    zx_str_free(cf->ctx, meta);
+    if (res_len)
+      *res_len = 1;
+    return zx_dup_cstr(cf->ctx, "n");
+  }
+  
+  if (auto_flags & (ZXID_AUTO_METAC | ZXID_AUTO_METAH)) {
+    if (auto_flags & ZXID_AUTO_METAH) {
+      ss = zx_strf(cf->ctx, "Content-Type: text/xml" CRLF "Content-Length: %d" CRLF2 "%.*s",
+		   meta->len, meta->len, meta->s);
+      zx_str_free(cf->ctx, meta);
+    } else
+      ss = meta;
+    res = ss->s;
+    if (res_len)
+      *res_len = ss->len;
+    ZX_FREE(cf->ctx, ss);
+    return res;
+  }
+  zx_str_free(cf->ctx, meta);
+  if (res_len)
+    *res_len = 1;
+  return zx_dup_cstr(cf->ctx, "b");
+}
+
+/* NULL return means the not logged in processing is needed, see zxid_simple_no_ses_cf() */
+
+char* zxid_simple_ses_active_cf(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, int* res_len, int auto_flags)
+{
+  char* post_str;
   char* res = 0;
   struct zx_str* ss;
-  struct zx_str* meta;
-  struct zxid_entity* idp;
-  struct zxid_cgi cgi;
-  struct zxid_ses ses;
-  memset(&cgi, 0, sizeof(cgi));
-  memset(&ses, 0, sizeof(ses));
   
-  if (!cf) {
-    ERR("NULL pointer. You MUST supply configuration object %d", auto_flags);
+  if (!cf || !cgi || !ses) {
+    ERR("FATAL: NULL pointer. You MUST supply configuration(%p), cgi(%p), and session(%p) objects (programming error)", cf, cgi, ses);
     exit(1);
   }
   
-  /*fprintf(stderr, "qs(%s) arg, autoflags=%x\n", qs, auto_flags);*/
-  zxid_set_opt(cf, 1, auto_flags & ZXID_AUTO_DEBUG?1:0);
+  if (!cgi->sid || !zxid_get_ses(cf, ses, cgi->sid)) {
+    D("No session(%s) active op(%c)", STRNULLCHK(cgi->sid), cgi->op);
+    return 0;
+  }
   
-  if (!qs) {
-    qs = getenv("QUERY_STRING");
-    D("QUERY_STRING(%s)", qs);
-    if (qs) {
-      zxid_parse_cgi(&cgi, qs);
-      if (cgi.op == 'P') {
-	cont_len = getenv("CONTENT_LENGTH");
-	if (cont_len) {
-	  sscanf(cont_len, "%d", &got);
-	  buf = ZX_ALLOC(cf->ctx, got);
-	  if (!buf) {
-	    ERR("out of memory len=%d", got);
-	    exit(1);
-	  }
-	  if (read_all_fd(0, buf, got, &got) == -1) {
-	    perror("Trouble reading post content.");
-	  } else {
-	    buf[got] = 0;
-	    D("POST(%s) got=%d cont_len(%s)", buf, got, cont_len);
-	    if (buf[0] == '<') goto sp_soap;                      /* No BOM and looks like XML */
-	    if (buf[2] == '<') { got-=2; buf+=2; ERR("UTF-16 NOT SUPPORTED %x%x", buf[0], buf[1]); goto sp_soap; }  /* UTF-16 BOM and looks XML */
-	    if (buf[3] == '<') { got-=3; buf+=3;                  /* UTF-8 BOM and looks XML */
-	    sp_soap:
-	      /* *** TODO: SOAP response should not be sent internally unless there is auto */
-	      ret = zxid_sp_soap_parse(cf, &cgi, &ses, got, buf);
-	      D("POST soap parse returned %d", ret);
-	      if (ret == ZXID_SSO_OK)
-		goto show_protected_content;
-	      goto soap_ret;
-	    }
-	    zxid_parse_cgi(&cgi, buf);
-	  }
-	}
+  /* OPs (the o= CGI field. Not to be confused with first letter of zxid_simple() return value)
+   * l = local logout (form gl)
+   * r = SLO redir    (form gr)
+   * s = SLO soap     (form gs)
+   * t = nireg redir  (form gt)
+   * u = nireg soap   (form gu)
+   * m = Show management screen
+   * n = Just check session (used for checking session for protected content pages)
+   * P = POST response. HTTP POST in general
+   * Q = POST request
+   * S = SOAP (POST) request
+   * B = Metadata
+   *
+   * M = CDC redirect and LECP detect
+   * C = CDC reader
+   * E = Normal "Entry" page (e.g. after CDC read)
+   * L = Start SSO (submit of E)
+   * A = Artifact processing
+   */
+  
+  if (cgi->enc_hint)
+    cf->nameid_enc = cgi->enc_hint != '0';
+  D("session(%s) active op(%c)", cgi->sid, cgi->op);
+  switch (cgi->op) {
+  case 'l':
+    if (cf->log_level>0)
+      zxlog(cf, 0,0,0,0,0,0, ses->nameid?ses->nameid->gg.content:0, "N", "W", "LOCLO", ses->sid,0);
+    zxid_del_ses(cf, ses);
+    cgi->msg = "Local logout Ok. Session terminated.";
+    return zxid_simple_show_login(cf, cgi, res_len, auto_flags);
+  case 'r':
+    if (auto_flags & ZXID_AUTO_REDIR) {
+      zxid_sp_slo_redir(cf, cgi, ses);
+      zxid_del_ses(cf, ses);
+      goto cgi_exit;
+    }
+    ss = zxid_sp_slo_location(cf, cgi, ses);
+    zxid_del_ses(cf, ses);
+    goto res_zx_str;
+  case 's':
+    zxid_sp_slo_soap(cf, cgi, ses);
+    zxid_del_ses(cf, ses);
+    cgi->msg = "SP Initiated logout (SOAP). Session terminated.";
+    return zxid_simple_show_login(cf, cgi, res_len, auto_flags);
+  case 't':
+    if (auto_flags & ZXID_AUTO_REDIR) {
+      zxid_sp_mni_redir(cf, cgi, ses, zx_ref_str(cf->ctx, cgi->newnym));
+      goto cgi_exit;
+    }
+    ss = zxid_sp_mni_location(cf, cgi, ses, zx_ref_str(cf->ctx, cgi->newnym));
+    goto res_zx_str;
+  case 'u':
+    zxid_sp_mni_soap(cf, cgi, ses, zx_ref_str(cf->ctx, cgi->newnym));
+    cgi->msg = "SP Initiated defederation (SOAP).";
+    break;     /* Defederation does not have to mean SLO */
+  case 'm':
+    res = zxid_fed_mgmt_cf(cf, res_len, -1, cgi->sid, auto_flags);
+    if (auto_flags & ZXID_AUTO_EXIT)
+      exit(0);
+    return res;
+  case 'P':    /* POST Profile Responses */
+    post_str = cgi->saml_resp;
+    goto process_post;
+  case 'I':
+  case 'K':
+  case 'Q':    /* POST Profile Requests */
+    post_str = cgi->saml_req;
+process_post:
+    if (auto_flags & ZXID_AUTO_REDIR) {
+      switch (zxid_sp_dispatch(cf, cgi, ses, post_str)) {
+      case ZXID_OK:       return zxid_simple_show_login(cf, cgi, res_len, auto_flags);
+      case ZXID_REDIR_OK: goto cgi_exit;
+      }
+    } else {
+      ss = zxid_sp_dispatch_location(cf, cgi, ses, post_str);
+      switch (ss->s[0]) {
+      case 'K': return zxid_simple_show_login(cf, cgi, res_len, auto_flags);
+      case 'L': goto res_zx_str;
       }
     }
-  } else {
-    if (qs_len == -1)
-      qs_len = strlen(qs);
-    if (qs[qs_len]) {   /* *** reads one past end of buffer */
-      ERR("IMPLEMENTATION LIMIT: Query String MUST be nul terminated len=%d", qs_len);
-      exit(1);
-    }
-    zxid_parse_cgi(&cgi, qs);
+    D("Q err (fall thru) %d", 0);
+    break;
+  case 'B':  /* Metadata should work from logged in session, too. */
+    return zxid_simple_show_meta(cf, cgi, res_len, auto_flags);
+  case 'n': break;
   }
-  if (!cgi.op)
-    cgi.op = 'M';
-
-  /* Check if user already has a working session.
-   *** Add session cookie check. */
+  return zxid_simple_render_ses(cf, ses, res_len, auto_flags);
   
-  if (cgi.sid) {
-    D("sid(%s)", cgi.sid);
-    if (zxid_get_ses(cf, &ses, cgi.sid)) {
-      D("session active op(%c)", cgi.op);
-      switch (cgi.op) {
-      case 'l':
-	zxid_del_ses(cf, &ses);
-	cgi.msg = "Local logout Ok. Session terminated.";
-	goto show_login_ret;  /* Simply abandon local session. Falls thru to login screen. */
-      case 'r':
-	if (auto_flags & ZXID_AUTO_REDIR) {
-	  zxid_sp_slo_redir(cf, &cgi, &ses);
-	  zxid_del_ses(cf, &ses);
-	  goto cgi_exit;
-	}
-	ss = zxid_sp_slo_location(cf, &cgi, &ses);
-	zxid_del_ses(cf, &ses);
-	goto res_zx_str;
-      case 's':
-	zxid_sp_slo_soap(cf, &cgi, &ses);
-	zxid_del_ses(cf, &ses);
-	cgi.msg = "SP Initiated logout (SOAP). Session terminated.";
-	goto show_login_ret;  /* Falls thru to login screen. */
-      case 't':
-	if (auto_flags & ZXID_AUTO_REDIR) {
-	  zxid_sp_nireg_redir(cf, &cgi, &ses, 0);
-	  goto cgi_exit;
-	}
-	ss = zxid_sp_nireg_location(cf, &cgi, &ses, 0);
-	goto res_zx_str;
-      case 'u':
-	zxid_sp_nireg_soap(cf, &cgi, &ses, 0);
-	cgi.msg = "SP Initiated defederation (SOAP).";
-	break;     /* Defederation does not have to mean SLO */
-      case 'P':    /* POST Profile Responses */
-	if (auto_flags & ZXID_AUTO_REDIR) {
-	  switch (zxid_sp_dispatch(cf, &cgi, &ses, cgi.saml_resp)) {
-	  case ZXID_OK:       goto show_login_ret;
-	  case ZXID_REDIR_OK: goto cgi_exit;
-	  }
-	} else {
-	  ss = zxid_sp_dispatch_location(cf, &cgi, &ses, cgi.saml_resp);
-	  switch (ss->s[0]) {
-	  case 'K': goto show_login_ret;
-	  case 'L': goto res_zx_str;
-	  }
-	}
-	break;
-      case 'Q':    /* POST Profile Requests */
-	if (auto_flags & ZXID_AUTO_REDIR) {
-	  switch (zxid_sp_dispatch(cf, &cgi, &ses, cgi.saml_req)) {
-	  case ZXID_OK:       goto show_login_ret;
-	  case ZXID_REDIR_OK: goto cgi_exit;
-	  }
-	} else {
-	  ss = zxid_sp_dispatch_location(cf, &cgi, &ses, cgi.saml_req);
-	  switch (ss->s[0]) {
-	  case 'K': goto show_login_ret;
-	  case 'L': goto res_zx_str;
-	  }
-	}
-	break;
-      case 'B':  /* Metadata should work from logged in session, too. */
-	goto metadata;
-      }
+cgi_exit:
+  if (auto_flags & ZXID_AUTO_EXIT)
+    exit(0);
+  res = zx_dup_cstr(cf->ctx, "n");
+  if (res_len)
+    *res_len = 1;
+  return res;
 
-    show_protected_content:
-      D("show_protected_content %d", 0);
-      ss = zxid_ses_to_ldif(cf, &ses);
-      D("LDIF(%.*s)", ss?ss->len:1, ss?ss->s:"-");
-      goto res_zx_str;
-    }
+res_zx_str:
+  res = ss->s;
+  if (res_len)
+    *res_len = ss->len;
+  ZX_FREE(cf->ctx, ss);
+  return res;
+}
+
+char* zxid_simple_no_ses_cf(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, int* res_len, int auto_flags)
+{
+  char* post_str;
+  char* res = 0;
+  struct zx_str* ss;
+  
+  if (!cf || !cgi || !ses) {
+    ERR("FATAL: NULL pointer. You MUST supply configuration(%p), cgi(%p), and session(%p) objects (programming error)", cf, cgi, ses);
+    exit(1);
   }
 
-  /* No session yet! Process login form */
-
-  memset(&ses, 0, sizeof(ses));
-  
-  switch (cgi.op) {
+  switch (cgi->op) {
   case 'M':  /* Invoke LECP or redirect to CDC reader. */
-    ss = zxid_lecp_check(cf, &cgi);
+    ss = zxid_lecp_check(cf, cgi);
     D("LECP check: ss(%.*s)", ss?ss->len:1, ss?ss->s:"?");
     if (ss) {
       if (auto_flags & ZXID_AUTO_REDIR) {
@@ -563,12 +761,11 @@ char* zxid_simple_cf(struct zxid_conf* cf, int qs_len, char* qs, int* res_len, i
 	goto res_zx_str;
       }
     }
-#if 0
   case 'C':  /* CDC Read: Common Domain Cookie Reader */
-    return zxid_cdc_read(cf, &cgi);
-#endif
+    zxid_cdc_read(cf, cgi);
+    goto cgi_exit;
   case 'E':  /* Return from CDC read, or start here to by-pass CDC read. */
-    ss = zxid_lecp_check(cf, &cgi);  /* use o=E&fc=1&fn=p  to set allow create true */
+    ss = zxid_lecp_check(cf, cgi);  /* use o=E&fc=1&fn=p  to set allow create true */
     D("LECP check: ss(%.*s)", ss?ss->len:1, ss?ss->s:"?");
     if (ss) {
       if (auto_flags & ZXID_AUTO_REDIR) {
@@ -578,118 +775,192 @@ char* zxid_simple_cf(struct zxid_conf* cf, int qs_len, char* qs, int* res_len, i
       } else
 	goto res_zx_str;
     }
-#if 0
-    if (zxid_cdc_check(cf, &cgi))
+    if (zxid_cdc_check(cf, cgi))
       return 0;
-#endif
+    D("NOT CDC %d", 0);
     break;
   case 'L':
     if (auto_flags & ZXID_AUTO_REDIR) {
-      zxid_start_sso(cf, &cgi);
+      zxid_start_sso(cf, cgi);
       goto cgi_exit;
     } else {
-      ss = zxid_start_sso_location(cf, &cgi);
+      ss = zxid_start_sso_location(cf, cgi);
       goto res_zx_str;
     }
     break;
   case 'A':
-    D("Process artifact(%s) pid=%d", cgi.saml_art, getpid());
-    switch (zxid_sp_deref_art(cf, &cgi, &ses)) {
+    D("Process artifact(%s) pid=%d", cgi->saml_art, getpid());
+    switch (zxid_sp_deref_art(cf, cgi, ses)) {
     case ZXID_REDIR_OK: ERR("*** Odd, redirect on artifact deref. %d", 0); break;
-    case ZXID_SSO_OK:   goto show_protected_content;
+    case ZXID_SSO_OK:
+show_protected_content_setcookie:
+      D("show_protected_content_setcookie: (%s)", cf->ses_cookie_name);
+      if (cf->ses_cookie_name && *cf->ses_cookie_name) {
+	ses->setcookie = zx_alloc_sprintf(cf->ctx, 0, "%s=%s; path=/; secure", cf->ses_cookie_name, ses->sid);
+	ses->cookie = zx_alloc_sprintf(cf->ctx, 0, "$Version=1; %s=%s", cf->ses_cookie_name, ses->sid);
+      }
+      ses->rs = cgi->rs;
+      return zxid_simple_render_ses(cf, ses, res_len, auto_flags);
     }
     break;
   case 'P':    /* POST Profile Responses */
-    D("Process response(%s)", cgi.saml_resp);
-    if (auto_flags & ZXID_AUTO_REDIR) {
-      switch (zxid_sp_dispatch(cf, &cgi, &ses, cgi.saml_resp)) {
-      case ZXID_SSO_OK:   goto show_protected_content;
-      case ZXID_REDIR_OK: goto cgi_exit;
-      }
-    } else {
-      ss = zxid_sp_dispatch_location(cf, &cgi, &ses, cgi.saml_resp);
-      switch (ss->s[0]) {
-      case 'M': goto show_protected_content;
-      case 'L': goto res_zx_str;
-      }
-    }
-    break;
+    DD("Process response(%s)", cgi->saml_resp);
+    DD("Process response. %d", 0);
+    post_str = cgi->saml_resp;
+    goto process_post;
+  case 'I':
+  case 'K':
   case 'Q':    /* POST Profile Requests */
-    D("Process request(%s)", cgi.saml_req);
+    DD("Process request(%s)", cgi->saml_req);
+    post_str = cgi->saml_req;
+process_post:
     if (auto_flags & ZXID_AUTO_REDIR) {
-      switch (zxid_sp_dispatch(cf, &cgi, &ses, cgi.saml_req)) {
-      case ZXID_SSO_OK:   goto show_protected_content;
+      switch (zxid_sp_dispatch(cf, cgi, ses, post_str)) {
+      case ZXID_SSO_OK:   goto show_protected_content_setcookie;
       case ZXID_REDIR_OK: goto cgi_exit;
       }
     } else {
-      ss = zxid_sp_dispatch_location(cf, &cgi, &ses, cgi.saml_req);
+      ss = zxid_sp_dispatch_location(cf, cgi, ses, post_str);
+      D("POST dispatch_loc(%s)", ss->s);
       switch (ss->s[0]) {
-      case 'M': goto show_protected_content;
+      case 'O': goto show_protected_content_setcookie;
+      case 'M': return zxid_simple_render_ses(cf, ses, res_len, auto_flags);
       case 'L': goto res_zx_str;
       }
     }
+    D("Q err (fall thru) %d", 0);
     break;
   case 'B':  /* Metadata */
-  metadata:
-    meta = zxid_sp_meta(cf, &cgi);
-    if (auto_flags & ZXID_AUTO_METAC && auto_flags & ZXID_AUTO_METAH) {
-      printf("Content-Type: text/xml" CRLF "Content-Length: %d" CRLF2 "%.*s",
-	     meta->len, meta->len, meta->s);
-      goto cgi_exit;
-    } else {
-      if (auto_flags & (ZXID_AUTO_METAC | ZXID_AUTO_METAH)) {
-	if (auto_flags & ZXID_AUTO_METAH) {
-	  ss = zx_strf(cf->ctx, "Content-Type: text/xml" CRLF "Content-Length: %d" CRLF2 "%.*s",
-		       meta->len, meta->len, meta->s);
-	  zx_str_free(cf->ctx, meta);
-	} else
-	  ss = meta;
-	goto res_zx_str;
-      }
-      res = zx_dup_cstr(cf->ctx, "b");
-      goto done;
-    }
-  default: D("unknown op(%c)", cgi.op);
+    return zxid_simple_show_meta(cf, cgi, res_len, auto_flags);
+  default: D("unknown op(%c)", cgi->op);
   }
-  
-show_login_ret:
-  D("show_login_ret %p", cf);
-  meta = zxid_idp_select_zxstr_cf(cf, auto_flags);
-  DD("idp_select: ret(%s)", meta?meta->len:1, meta?meta->s:"?");
-  if (auto_flags & ZXID_AUTO_LOGINC && auto_flags & ZXID_AUTO_LOGINH) {  /* Both H&C: CGI */
-    printf("Content-Type: text/html" CRLF "Content-Length: %d" CRLF2 "%.*s",
-	   meta->len, meta->len, meta->s);
-    goto cgi_exit;
-  } else {
-    if (auto_flags & (ZXID_AUTO_LOGINC | ZXID_AUTO_LOGINH)) {
-      if (auto_flags & ZXID_AUTO_LOGINH) {  /* H only: return both H and C */
-	ss = zx_strf(cf->ctx, "Content-Type: text/xml" CRLF "Content-Length: %d" CRLF2 "%.*s",
-		     meta->len, meta->len, meta->s);
-	zx_str_free(cf->ctx, meta);
-      } else
-	ss = meta;       /* C only */
-      goto res_zx_str;
-    }
-    res = zx_dup_cstr(cf->ctx, "e");   /* Neither H nor C */
-    goto done;
-  }
-  
-soap_ret:
-  if (auto_flags & ZXID_AUTO_SOAPC || auto_flags & ZXID_AUTO_SOAPH)
-    goto cgi_exit;
-  res = zx_dup_cstr(cf->ctx, ret ? "n" : "*** SOAP error (enable debug if you want to see why)"); 
-  goto done;
+  return zxid_simple_show_login(cf, cgi, res_len, auto_flags);
 
 cgi_exit:
   if (auto_flags & ZXID_AUTO_EXIT)
     exit(0);
   res = zx_dup_cstr(cf->ctx, "n");
-  goto done;
+  if (res_len)
+    *res_len = 1;
+  return res;
 
 res_zx_str:
   res = ss->s;
+  if (res_len)
+    *res_len = ss->len;
   ZX_FREE(cf->ctx, ss);
-  goto done;
+  return res;
+}
+
+
+/* Simple handler that assumes the configuration has already been read in.
+ * The memory for result is grabbed from ZX_ALLOC(), usually malloc(3)
+ * and is "given" away to the caller, i.e. caller must free it. The
+ * return value is LDIF of attributes in success case.
+ * res_len, if non-null, will receive the length of the response. */
+
+/* Called by:  zxid_simple_len */
+char* zxid_simple_cf(struct zxid_conf* cf, int qs_len, char* qs, int* res_len, int auto_flags)
+{
+  int got, ret;
+  char* remote_addr;
+  char* cont_len;
+  char* buf = 0;
+  char* res = 0;
+  struct zxid_cgi cgi;
+  struct zxid_ses ses;
+  memset(&cgi, 0, sizeof(cgi));
+  memset(&ses, 0, sizeof(ses));
+  
+  if (!cf) {
+    ERR("NULL pointer. You MUST supply configuration object %d  (programming error)", auto_flags);
+    exit(1);
+  }
+  
+  /*fprintf(stderr, "qs(%s) arg, autoflags=%x\n", qs, auto_flags);*/
+  zxid_set_opt(cf, 1, auto_flags & ZXID_AUTO_DEBUG?1:0);
+  if (!cf->ipport) {
+    remote_addr = getenv("REMOTE_ADDR");
+    if (remote_addr) {
+      cf->ipport = ZX_ALLOC(cf->ctx, strlen(remote_addr) + 6 + 1); /* :12345 */
+      sprintf(cf->ipport, "%s:-", remote_addr);
+    }
+  }
+  
+  if (!qs) {
+    qs = getenv("QUERY_STRING");
+    D("QUERY_STRING(%s)", qs);
+    if (qs) {
+      zxid_parse_cgi(&cgi, qs);
+      if (ONE_OF_2(cgi.op, 'P', 'S')) {
+	cont_len = getenv("CONTENT_LENGTH");
+	if (cont_len) {
+	  sscanf(cont_len, "%d", &got);
+	  buf = ZX_ALLOC(cf->ctx, got);
+	  if (!buf) {
+	    ERR("out of memory len=%d", got);
+	    exit(1);
+	  }
+	  if (read_all_fd(0, buf, got, &got) == -1) {
+	    perror("Trouble reading post content.");
+	  } else {
+	    buf[got] = 0;
+	    D("POST(%s) got=%d cont_len(%s)", buf, got, cont_len);
+	    if (buf[0] == '<') goto sp_soap;  /* No BOM and looks like XML */
+	    if (buf[2] == '<') {              /* UTF-16 BOM and looks like XML */
+	      got-=2; buf+=2;
+	      ERR("UTF-16 NOT SUPPORTED %x%x", buf[0], buf[1]);
+	      goto sp_soap;
+	    }
+	    if (buf[3] == '<') {              /* UTF-8 BOM and looks XML */
+	      got-=3; buf+=3;
+	    sp_soap:
+	      /* *** TODO: SOAP response should not be sent internally unless there is auto */
+	      ret = zxid_sp_soap_parse(cf, &cgi, &ses, got, buf);
+	      D("POST soap parse returned %d", ret);
+	      if (ret == ZXID_SSO_OK)
+		return zxid_simple_render_ses(cf, &ses, res_len, auto_flags);
+	      if (auto_flags & ZXID_AUTO_SOAPC || auto_flags & ZXID_AUTO_SOAPH) {
+		if (auto_flags & ZXID_AUTO_EXIT)
+		  exit(0);
+		res = zx_dup_cstr(cf->ctx, "n");
+		if (res_len)
+		  *res_len = 1;
+		goto done;
+	      }
+	      res = zx_dup_cstr(cf->ctx, ret ? "n" : "*** SOAP error (enable debug if you want to see why)"); 
+	      if (res_len)
+		*res_len = strlen(res);
+	      goto done;
+	    }
+	    zxid_parse_cgi(&cgi, buf);
+	  }
+	}
+      }
+    }
+  } else {
+    if (qs_len == -1)
+      qs_len = strlen(qs);
+    if (qs[qs_len]) {   /* *** reads one past end of buffer */
+      ERR("IMPLEMENTATION LIMIT: Query String MUST be nul terminated len=%d", qs_len);
+      exit(1);
+    }
+    D("QUERY_STRING(%s)", qs);
+    zxid_parse_cgi(&cgi, qs);
+  }
+  if (!cgi.op)
+    cgi.op = 'M';  /* By default, if no ses, check CDC and offer SSO */
+
+  if (!cgi.sid && cf->ses_cookie_name && *cf->ses_cookie_name)
+    zxid_get_sid_from_cookie(cf, &cgi, getenv("HTTP_COOKIE"));
+
+  if (cgi.sid) {
+    if (res = zxid_simple_ses_active_cf(cf, &cgi, &ses, res_len, auto_flags))
+      goto done;
+  }
+
+  memset(&ses, 0, sizeof(ses));   /* No session yet! Process login form */
+  res = zxid_simple_no_ses_cf(cf, &cgi, &ses, res_len, auto_flags);
 
 done:
   if (buf)
@@ -703,7 +974,11 @@ done:
 /* Called by:  zxid_simple */
 char* zxid_simple_len(int conf_len, char* conf, int qs_len, char* qs, int* res_len, int auto_flags)
 {
+  struct zx_ctx ctx;
   struct zxid_conf cf;
+  zx_reset_ctx(&ctx);
+  memset(&cf, 0, sizeof(struct zxid_conf));
+  cf.ctx = &ctx;
   zxid_conf_to_cf_len(&cf, conf_len, conf);
   return zxid_simple_cf(&cf, qs_len, qs, res_len, auto_flags);
 }
