@@ -1,22 +1,23 @@
 /* zxlib.c  -  Utility functions for generated (and other) code
- * Copyright (c) 2006 Symlabs (symlabs@symlabs.com), All Rights Reserved.
+ * Copyright (c) 2006-2007 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing.
  * Licensed under Apache License 2.0, see file COPYING.
- * $Id: zxlib.c,v 1.28 2007-08-10 19:19:10 sampo Exp $
+ * $Id: zxlib.c,v 1.37 2008-05-26 15:28:44 sampo Exp $
  *
  * 28.5.2006, created --Sampo
  * 8.8.2006,  moved lookup functions to generated code --Sampo
  * 12.8.2006, added special scanning of xmlns to avoid backtracking elem recognition --Sampo
- * 26.8.2006, significatn Common Subexpression Elimination (CSE) --Sampo
+ * 26.8.2006, significant Common Subexpression Elimination (CSE) --Sampo
+ * 30.9.2007, more CSE --Sampo
  */
 
 //#include <pthread.h>
-#ifndef MACOSX
-#include <malloc.h>
-#endif
+//#ifndef MACOSX
+//#include <malloc.h>
+//#endif
 #include <memory.h>
 #include <string.h>
 #include <stdarg.h>
@@ -95,6 +96,7 @@ void zx_str_conv(struct zx_str* ss, int* out_len, char** out_s)  /* SWIG typemap
   *out_len = ss->len;
 }
 
+/* Free both the zx_str node and the underlying string data */
 /* Called by:  zxid_idp_list_cf, zxid_lecp_check, zxid_saml2_location, zxid_saml2_redir, zxid_saml2_redir_url, zxid_saml2_resp_location, zxid_saml2_resp_redir, zxid_send_sp_meta, zxid_simple_cf x4, zxid_snarf_eprs_from_ses x2, zxid_start_sso_location, zxid_write_ent_to_cache */
 void zx_str_free(struct zx_ctx* c, struct zx_str* ss)
 {
@@ -115,17 +117,27 @@ struct zx_str* zx_ref_len_str(struct zx_ctx* c, int len, char* s)
 /* Called by:  zxid_ac_desc x2, zxid_issuer, zxid_key_desc x2, zxid_mk_Status x2, zxid_mk_art_deref x2, zxid_mk_authn_req x8, zxid_mk_di_req_svc x2, zxid_mk_ecp_Request_hdr x2, zxid_mk_logout x2, zxid_mk_logout_resp x2, zxid_mk_nireg, zxid_mk_nireg_resp, zxid_mk_paos_Request_hdr x3, zxid_nireg_desc, zxid_slo_desc, zxid_sp_sso_desc x3 */
 struct zx_str* zx_ref_str(struct zx_ctx* c, char* s)
 {
+  if (!s)
+    return 0;
   return zx_ref_len_str(c, strlen(s), s);
 }
 
+/* Newly allocated string (node and data) of specified length, but uninitialized */
+
+struct zx_str* zx_new_len_str(struct zx_ctx* c, int len)
+{
+  struct zx_str* ss = ZX_ZALLOC(c, struct zx_str);
+  ss->s = ZX_ALLOC(c, len+1);
+  ss->s[len] = 0;
+  ss->len = len;
+  return ss;
+}
+
 /* Called by:  zx_dup_len_simple_elem, zx_dup_str */
 struct zx_str* zx_dup_len_str(struct zx_ctx* c, int len, char* s)
 {
-  struct zx_str* ss = ZX_ZALLOC(c, struct zx_str);
-  ss->s = ZX_ALLOC(c, len+1);
+  struct zx_str* ss = zx_new_len_str(c, len);
   memcpy(ss->s, s, len);
-  ss->s[len] = 0;
-  ss->len = len;
   return ss;
 }
 
@@ -135,24 +147,49 @@ struct zx_str* zx_dup_str(struct zx_ctx* c, char* s)
   return zx_dup_len_str(c, strlen(s), s);
 }
 
+char* zx_alloc_vasprintf(struct zx_ctx* c, int *retlen, char* f, va_list ap) /* data is new memory */
+{
+  va_list ap2;
+  int len;
+  char* s;
+  char buf[2]; 
+  va_copy(ap2, ap);
+  len = vsnprintf(buf, 1, f, ap2);
+  va_end(ap2);
+  if (len < 0) {
+    perror("vsnprintf");
+    D("Broken vsnprintf? Impossible to compute length of string. Be sure to `export LANG=C' if you get errors about multibyte characters. Length returned: %d", len);
+    if (retlen)
+      *retlen = 0;
+    s = ZX_ALLOC(c, 1);
+    s[0] = 0;
+    return s;
+  }
+  s = ZX_ALLOC(c, len+1);
+  vsnprintf(s, len+1, f, ap);
+  if (retlen)
+    *retlen = len;
+  return s;
+}
+
+char* zx_alloc_sprintf(struct zx_ctx* c, int *retlen, char* f, ...)  /* data is new memory */
+{
+  char* ret;
+  va_list ap;
+  va_start(ap, f);
+  ret = zx_alloc_vasprintf(c, retlen, f, ap);
+  va_end(ap);
+  return ret;
+}
+
 /* Called by:  zxid_ac_desc, zxid_date_time, zxid_fed_mgmt_cf x3, zxid_idp_list_cf, zxid_idp_select x3, zxid_lecp_check, zxid_mk_di_req_svc, zxid_mk_id, zxid_mk_paos_Request_hdr, zxid_my_cdc_url, zxid_my_entity_id, zxid_nireg_desc x2, zxid_saml2_location, zxid_saml2_redir_url, zxid_saml2_resp_location, zxid_saml_ok, zxid_ses_to_ldif, zxid_simple_cf x3, zxid_slo_desc x2, zxid_sp_sso_desc, zxid_start_sso_location */
 struct zx_str* zx_strf(struct zx_ctx* c, char* f, ...)  /* data is new memory */
 {
   va_list ap;
   int len;
   char* s;
-  char buf[2]; 
   va_start(ap, f);
-  len = vsnprintf(buf, 1, f, ap);
-  va_end(ap);
-  if (len < 0) {
-    perror("vsnprintf");
-    D("Broken vsnprintf? Impossible to compute length of string. Be sure to `export LANG=C' if you get errors about multibyte characters. Length returned: %d", len);
-    return zx_dup_str(c, "");
-  }
-  s = ZX_ALLOC(c, len+1);
-  va_start(ap, f);
-  vsnprintf(s, len+1, f, ap);
+  s = zx_alloc_vasprintf(c, &len, f, ap);
   va_end(ap);
   return zx_ref_len_str(c, len, s);
 }
@@ -196,365 +233,7 @@ struct zx_elem_s* zx_dup_simple_elem(struct zx_ctx* c, char* s)
   return zx_dup_len_simple_elem(c, strlen(s), s);
 }
 
-/* ------------- All the manner namespace management ------------- */
-
-/* Called by: */
-void zx_fix_any_elem_dec(struct zx_ctx* c, struct zx_any_elem_s* x, char* nam, int namlen)
-{
-  char* p = memchr(nam, ':', namlen);
-  if (p) {
-    x->gg.g.ns = zx_locate_ns_by_prefix(c, p-nam, nam);
-    ++p;
-    namlen -= p-nam;
-    nam = p;
-  }
-  x->name = nam;
-  x->name_len = namlen;
-}
-
-/* Called by: */
-int zx_is_ns_prefix(struct zx_ns_s* ns, int len, char* prefix)
-{
-  struct zx_ns_s* alias;
-  if (!ns)
-    return 0;
-  if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
-    return 1;
-  for (alias = ns->n; alias; alias = alias->n)
-    if (alias->prefix_len == len && (!len || !memcmp(alias->prefix, prefix, len)))
-      return 1;
-  return 0;
-}
-
-/* Called by:  zx_fix_any_elem_dec, zx_init_tok_tab, zx_xmlns_decl */
-struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, char* prefix)
-{
-  struct zx_ns_s* ns;
-  struct zx_ns_s* alias;
-  for (ns = c->ns_tab; ns->url_len; ++ns) {
-    if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
-      return ns;
-    for (alias = ns->n; alias; alias = alias->n)
-      if (alias->prefix_len == len && (!len || !memcmp(alias->prefix, prefix, len)))
-	return ns;
-  }
-  
-  /* Trailer element holds unknown namespaces. */
-  for (alias = ns->n; alias; alias = alias->n)
-    if (alias->prefix_len == len && (!len || !memcmp(alias->prefix, prefix, len)))
-      return alias;
-  return 0;
-}
-
-/* Called by:  zx_xmlns_decl */
-static struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, char* url)
-{
-  struct zx_ns_s* ns;
-  struct zx_ns_s* alias;
-  for (ns = c->ns_tab; ns->url_len; ++ns) {
-    if (ns->url_len == len && (!len || !memcmp(ns->url, url, len)))
-      return ns;
-    for (alias = ns->n; alias; alias = alias->n)
-      if (alias->url_len == len && (!len || !memcmp(alias->url, url, len)))
-	return alias;
-  }
-
-  /* Trailer element holds unknown namespaces. */
-  for (alias = ns->n; alias; alias = alias->n)
-    if (alias->url_len == len && (!len || !memcmp(alias->url, url, len)))
-      return alias;
-  return 0;
-}
-
-/* Called by:  zx_push_seen x2 */
-static struct zx_ns_s* zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len, char* url)
-{
-  struct zx_ns_s* alias;
-  struct zx_ns_s* ns = zx_locate_ns_by_url(c, url_len, url);
-  if (!ns) {
-    D("Namespace(%.*s) not found by URL(%.*s) (probably wrong namespace URL)", prefix_len, prefix, url_len, url);
-    ns = zx_locate_ns_by_prefix(c, prefix_len, prefix);
-    if (!ns) {
-      D("Namespace not found by prefix(%.*s) or URL(%.*s) (probably wrong namespace URL)", prefix_len, prefix, url_len, url);
-      return;  /* Namespace not known by compiled in schemata. */
-    }
-  }
-  /* Always alloc a new one because we may need to push to stack multiple instances of same. */
-  alias = ZX_ZALLOC(c, struct zx_ns_s);
-  alias->prefix_len = prefix_len;
-  alias->prefix = prefix;
-  alias->url_len = url_len;
-  alias->url =url;
-  alias->n = ns->n;
-  ns->n = alias;
-  return alias;
-}
-
-/* Called by:  zx_prefix_seen_whine, zx_push_seen */
-struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, char* prefix)
-{
-  struct zx_ns_s* ns;
-  for (ns = c->guard_seen_n.seen_n; ns->seen_n; ns = ns->seen_n)
-    if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
-      return ns;
-  return 0;
-}
-
-/* Called by:  TXattr_lookup, TXelem_lookup */
-struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, char* prefix, char* logkey)
-{
-  struct zx_ns_s* ns = zx_prefix_seen(c, len, prefix);
-  if (!ns) {
-    /* Try to locate the namespace by the prefix from among the namespaces
-     * naturally known to zxid. */
-    ns = zx_locate_ns_by_prefix(c, len, prefix);
-    if (!ns) {
-      D("Undefined namespace prefix(%.*s) at(%s). NS not known from any context.", len, prefix, logkey);
-    } else {
-      D("Undefined namespace prefix(%.*s) at(%s) mapped to uri(%.*s) by built-in table.", len, prefix, logkey, ns->url_len, ns->url);
-    }
-  }
-  return ns;
-}
-
-/* See if prefix has been seen, and in the same meaning. If not, the allocate
- * a new node and push or add it to the doubly linked list as well as to the
- * pop_seen list. Returns 0 if no addition was done (i.e. ns had been seen already). */
-
-/* Called by:  zx_add_xmlns_if_not_seen, zx_enc_xmlns_if_not_seen, zx_len_xmlns_if_not_seen, zx_scan_xmlns */
-static struct zx_ns_s* zx_push_seen(struct zx_ctx* c, int prefix_len, char* prefix, int url_len, char* url, struct zx_ns_s** pop_seen)
-{
-  struct zx_ns_s* old_ns;
-  struct zx_ns_s* ns;
-  old_ns = zx_prefix_seen(c, prefix_len, prefix);
-  if (old_ns) {
-    if (old_ns->url_len == url_len && (!url_len || !memcmp(old_ns->url, url, url_len)))
-      return 0;
-    ns = zx_xmlns_decl(c, prefix_len, prefix, url_len, url);
-    ns->seen = old_ns;            /* Push */
-    ns->seen_n = old_ns->seen_n;  /* Replace old_ns in middle of the list */
-    ns->seen_p = old_ns->seen_p;
-    old_ns->seen_n->seen_p = ns;
-    old_ns->seen_p->seen_n = ns;
-  } else {
-    ns = zx_xmlns_decl(c, prefix_len, prefix, url_len, url);
-    ns->seen_n = c->guard_seen_n.seen_n;  /* Add to beginning of seen_n list. */
-    c->guard_seen_n.seen_n = ns;
-    ns->seen_n->seen_p = ns;
-    ns->seen_p = &c->guard_seen_n;
-  }
-  ns->seen_pop = *pop_seen;
-  *pop_seen = ns;
-  return ns;
-}
-
-/* Called by:  TXDEC_ELNAME, TXENC_SO_ELNAME, TXENC_WO_ELNAME, TXLEN_SO_ELNAME, TXLEN_WO_ELNAME */
-void zx_pop_seen(struct zx_ns_s* ns)
-{
-  for (; ns; ns = ns->seen_pop) {
-    if (ns->seen) {
-      ns->seen->seen_n = ns->seen_n;   /* Replace ns with old_ns (ns->seen) in middle of list */
-      ns->seen->seen_p = ns->seen_p;
-      ns->seen_n->seen_p = ns->seen;
-      ns->seen_p->seen_n = ns->seen;
-    } else {
-      ns->seen_n->seen_p = ns->seen_p; /* Remove ns from middle of the seen_n list. */
-      ns->seen_p->seen_n = ns->seen_n;
-    }
-  }
-}
-
-/* When trying to scan an element, an annoying feature of XML namespaces is that the
- * namespace may be declared in a xmlns attribute within the element itself. Thus
- * at the time of scanning the <ns:element part we can't know its namespace. What
- * a lousy design. In order to handle this we need to either backtrack or
- * make a special case forward scan for xmlns attributes (which is redundant with
- * normal attribute scanning). It seems simpler to do the latter, so here goes...
- *
- * The return value represents the list of namespaces that were newly declared
- * at this level, i.e. pushed to the seen stacks. This list is used to pop the
- * seen stacks after we are through with the element. */
-
-/* Called by:  TXDEC_ELNAME */
-struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
-{
-  struct zx_ns_s* pop_seen = 0;  /* build a list of namespaces declared here */
-  char* prefix;
-  char* url;
-  char* p = c->p;  /* We need to keep the original c->p so normal attributes can be scanned. */
-  char quote;
-  int prefix_len, url_len;
-
-  /* The tag name has already been detected. Process attributes until '>' */
-  
-  for ( ; 1; ++p) {
-    ZX_SKIP_WS_P(c, p, pop_seen);
-    if (ONE_OF_2(*p, '>', '/'))
-      break;
-    if (!memcmp(p, "xmlns", sizeof("xmlns")-1)) {
-      switch (p[5]) {
-      case '=':  /* Default namespace decl. */
-	prefix = p;
-	goto scan_URL;
-      case ':':  /* Qualified namespace decl. */
-	prefix = p += 6;
-	ZX_LOOK_FOR_P(c,'=',p, pop_seen);
-      scan_URL:
-	++p;
-	if (!ONE_OF_2(*p, '"', '\''))
-	  return pop_seen;
-	quote = *p;
-	url = ++p;
-	ZX_LOOK_FOR_P(c, quote, p, pop_seen);
-
-	prefix_len = (url - 2) - prefix;
-	url_len = p - url;
-	zx_push_seen(c, prefix_len, prefix, url_len, url, &pop_seen);
-	goto next;
-      default:
-	D("Illformed attributes. Bad char(%c)", p[5]);
-	return pop_seen;
-      }
-    }
-    
-    /* Skip over any other attributes. */
-    
-    ZX_LOOK_FOR_P(c,'=',p, pop_seen);
-    ++p;
-    if (!ONE_OF_2(*p, '"', '\''))
-      return pop_seen;
-    quote = *p;
-    ++p;
-    ZX_LOOK_FOR_P(c,quote,p, pop_seen);
-  next:
-    continue;
-  }
-  return pop_seen;
-}
-
-/* Disambiguate token by considering its namespace.
- * See zx_attr_lookup(), zx_elem_lookup() */
-/* Called by:  TXattr_lookup, TXelem_lookup */
-const struct zx_tok* zx_tok_by_ns(const struct zx_tok* zt, const struct zx_tok* lim,
-				  int len, char* name, struct zx_ns_s* ns)
-{
-  struct zx_ns_s* alias;
-  const struct zx_tok* ztt;
-
-  /* First find token whose name matches. The token table CAN have duplicate entries,
-   * see -D flag to gperf. */
-  for (; zt < lim && (memcmp(zt->name, name, len) || zt->name[len]); ++zt) ;
-  ztt = zt < lim ? zt : 0;  /* Remember first "hit" in case we fail. */
-  if (!ns)
-    return ztt;
-  
-  /* Now check for namespace match. */
-  for (; zt < lim && (!memcmp(zt->name, name, len) && !zt->name[len]); ++zt)
-    for (alias = zt->ns; alias; alias = alias->n)
-      if (alias == ns)
-	return zt;
-  return ztt;
-}
-
-#if 0
-/* N.B. Generally this function is not needed since tokens can be arranged
- * to point to respective struct zx_ns_s at compile time using static
- * initialization. This is handled by xsd2sg.pl */
-/*if (!zx_init_tok_tab(&ctx, zx_elems, zx_elems + sizeof(zx_elems) / sizeof(struct zx_tok)))  DIE("Inconsistency between tokens and namespaces");*/
-/* Called by: */
-int zx_init_tok_tab(struct zx_ctx* c, struct zx_tok* tok_tab, struct zx_tok* tok_tab_lim)
-{
-  for (; tok_tab < tok_tab_lim; ++tok_tab)
-    if (!(tok_tab->ns = zx_locate_ns_by_prefix(c, strlen(tok_tab->prefix), (char*)tok_tab->prefix)))
-      return 0;
-  return 1;
-}
-#endif
-
-/* Called by: */
-int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
-{
-  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
-    return 0;
-  return sizeof(" xmlns")-1
-    + (ns->prefix_len ? ns->prefix_len+1 : 0) + 2 + ns->url_len + 1;
-}
-
-/* Called by: */
-char* zx_enc_xmlns_if_not_seen(struct zx_ctx* c, char* p, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
-{
-  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
-    return p;
-  
-  ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
-  if (ns->prefix_len) {
-    ZX_OUT_CH(p, ':');
-    ZX_OUT_MEM(p, ns->prefix, ns->prefix_len);
-  }
-  ZX_OUT_CH(p, '=');
-  ZX_OUT_CH(p, '"');
-  ZX_OUT_MEM(p, ns->url, ns->url_len);
-  ZX_OUT_CH(p, '"');
-  return p;
-}
-
-/* For WO encoder the sort of xmlns declarations is not known at compile
- * time. Thus we first add them to the pop_seen list using insertion
- * sort (pop_seen is smallest and prefixes grow from there) and
- * then later render the list using zx_enc_seen(). */
-
-/* Called by: */
-void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
-{
-  struct zx_ns_s* pop_seen_dummy=0;
-  struct zx_ns_s* new_ns;
-  int res;
-  new_ns = zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, &pop_seen_dummy);
-  if (!new_ns)
-    return;
-  if (!*pop_seen) {
-    *pop_seen = new_ns;
-    return;
-  }
-  if (!new_ns->prefix_len) {       /* Default namespace (empty prefix) sorts first. */
-first:
-    new_ns->seen_pop = *pop_seen;
-    *pop_seen = new_ns;
-    return;
-  }
-
-  ns = *pop_seen;
-  if (ns->prefix_len) {
-    res = memcmp(ns->prefix, new_ns->prefix, MIN(ns->prefix_len, new_ns->prefix_len));
-    if ((res > 0) || !res && ns->prefix_len >= new_ns->prefix_len)
-      goto first;
-  }
-  for (; ns->seen_pop; ns = ns->seen_pop) {
-    res = memcmp(ns->seen_pop->prefix, new_ns->prefix,
-		 MIN(ns->seen_pop->prefix_len, new_ns->prefix_len));
-    if ((res > 0) || !res && ns->seen_pop->prefix_len >= new_ns->prefix_len)
-      break;
-  }
-  new_ns->seen_pop = ns->seen_pop;
-  ns->seen_pop = new_ns;
-}
-
-/* Called by:  TXENC_WO_ELNAME */
-char* zx_enc_seen(char* p, struct zx_ns_s* ns)
-{
-  for (; ns; ns = ns->seen_pop) {
-    ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
-    if (ns->prefix_len) {
-      ZX_OUT_CH(p, ':');
-      ZX_OUT_MEM(p, ns->prefix, ns->prefix_len);
-    }
-    ZX_OUT_CH(p, '=');
-    ZX_OUT_CH(p, '"');
-    ZX_OUT_MEM(p, ns->url, ns->url_len);
-    ZX_OUT_CH(p, '"');
-  }
-  return p;
-}
+/* ------------- Common Subexpression Elimination for generated code ------------- */
 
 /* Render the unknown attributes list. CSE for almost all tags. */
 
@@ -573,7 +252,7 @@ int zx_len_so_common(struct zx_ctx* c, struct zx_elem_s* x)
   }
 
   for (ae = x->any_elem; ae; ae = (struct zx_any_elem_s*)ae->gg.g.n)    /* unknown elements */
-    len += zx_LEN_SO_simple_elem(c, &ae->gg, ae->name_len, 0);          /* *** figure namespaces */
+    len += zx_LEN_SO_simple_elem(c, &ae->gg, ae->name_len, ae->gg.g.ns);
   
   for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n)             /* content */
     len += ss->len;
@@ -654,7 +333,7 @@ struct zx_str* zx_easy_enc_common(struct zx_ctx* c, char* p, char* buf, int len)
 int zx_attr_so_len(struct zx_str* attr, int name_size)
 {
   int len = 0;
-  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  /* In legal XML there should really be just one attribute, but we accommodate multioccurances. */
   for (; attr; attr = (struct zx_str*)attr->g.n)
     len += 1 + name_size + 1 + 1 + attr->len + 1;
   return len;
@@ -665,7 +344,7 @@ int zx_attr_so_len(struct zx_str* attr, int name_size)
 /* Called by: */
 char* zx_attr_so_enc(char* p, struct zx_str* attr, char* name, int name_len)
 {
-  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  /* In legal XML there should really be just one attribute, but we accommodate multioccurances. */
   for (; attr; attr = (struct zx_str*)attr->g.n) {
     ZX_OUT_MEM(p, name, name_len);
     ZX_OUT_MEM(p, attr->s, attr->len);
@@ -678,7 +357,7 @@ char* zx_attr_so_enc(char* p, struct zx_str* attr, char* name, int name_len)
 int zx_attr_wo_len(struct zx_str* attr, int name_size)
 {
   int len = 0;
-  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  /* In legal XML there should really be just one attribute, but we accommodate multioccurances. */
   for (; attr; attr = (struct zx_str*)attr->g.n) {
     if (attr->g.ns && attr->g.ns->prefix_len)
       len += attr->g.ns->prefix_len + 1;
@@ -692,7 +371,7 @@ int zx_attr_wo_len(struct zx_str* attr, int name_size)
 /* Called by: */
 char* zx_attr_wo_enc(char* p, struct zx_str* attr, char* name, int name_len)
 {
-  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  /* In legal XML there should really be just one attribute, but we accommodate multioccurances. */
   for (; attr; attr = (struct zx_str*)attr->g.n) {
     ZX_OUT_CH(p, ' ');
     if (attr->g.ns && attr->g.ns->prefix_len) {
@@ -1020,4 +699,150 @@ void zx_prepare_dec_ctx(struct zx_ctx* c, struct zx_ns_s* ns_tab, char* start, c
   c->lim = lim;
 }
 
+int zx_scan_data(struct zx_ctx* c, struct zx_elem_s* el)
+{
+  struct zx_str* ss;
+  char* d = c->p;
+  if (c->p) ZX_LOOK_FOR(c,'<');
+  ss = ZX_ZALLOC(c, struct zx_str);
+  ss->len = c->p - d;
+  ss->s = d;
+  ss->g.tok = ZX_TOK_DATA;
+  ss->g.n = &el->content->g;
+  el->content = ss;
+  ss->g.wo = &el->kids->g;
+  el->kids = (struct zx_elem_s*)ss;
+  return 1;
+
+ look_for_not_found:
+  /*zx_xml_parse_err(c, '<', __FUNCTION__, "look for not found");  -- Causes bogus warning in end of buffer. */
+  return 0;
+}
+
+int zx_scan_pi_or_comment(struct zx_ctx* c)
+{
+  char* name;
+  char quote;
+
+  switch (*c->p) {
+  case '?':  /* processing instruction <?xml ... ?> */
+    name = c->p-1;
+    DD("Processing Instruction detected (%.*s)", 5, name);
+    while (1) {
+      quote = '>';
+      ZX_LOOK_FOR(c,'>');
+      if (c->p[-1] == '?')
+	break;
+    }
+    ++c->p;
+    DD("Processing Instruction scanned (%.*s)", c->p-name, name);
+    /*ZX_PI_DEC_EXT(pi);*/
+    return 0;
+  case '!':  /* comment <!-- ... --> */
+    name = c->p-1;
+    c->p += 2;
+    if (c->p[-1] != '-' || c->p[0] != '-') {
+      c->p -= 3;
+      return 1;
+    }
+    D("Comment detected (%.*s)", 8, name);
+    c->p += 2;
+    while (1) {
+      quote = '>';
+      ZX_LOOK_FOR(c,'>');
+      if (c->p[-2] == '-' && c->p[-1] == '-') {
+	break;
+      }
+    }
+    ++c->p;
+    D("Comment scanned (%.*s)", c->p-name, name);
+    /*ZX_COMMENT_DEC_EXT(comment);*/
+    return 0;
+  }
+  return 1;
+ look_for_not_found:
+  zx_xml_parse_err(c, quote, __FUNCTION__, "look for not found");
+  return 1;
+}
+
+struct zx_str* zx_dec_unknown_attr(struct zx_ctx* c, struct zx_elem_s* el, char* name, char* data, int tok, int ctx_tok)
+{
+  int namlen = data - name - 2;
+  struct zx_any_attr_s* attr = ZX_ZALLOC(c, struct zx_any_attr_s);
+  
+  if (tok == ZX_TOK_NOT_FOUND) {
+    D("Unknown attribute(%.*s) in context(%d)", namlen, name, ctx_tok);
+  } else {
+    D("Known attribute(%.*s) tok(%d) in wrong context(%d)", namlen, name, tok, ctx_tok);
+    tok = ZX_TOK_NOT_FOUND;
+  }
+
+  attr->name_len = namlen;
+  attr->name = name;
+  attr->ss.g.n = &el->any_attr->ss.g;
+  el->any_attr = attr;
+  /* *** namespace handling for unknown attribute? */
+  /*ZX_UNKNOWN_ATTR_DEC_EXT(attr);*/
+  return &attr->ss;
+}
+
+char* zx_dec_attr_val(struct zx_ctx* c, char** name)
+{
+  char* data;
+  char quote;
+  *name = c->p;
+  quote = '=';
+  ZX_LOOK_FOR(c,'=');
+  
+  ++c->p;
+  if (!ONE_OF_2(*c->p, '"', '\'')) {
+    zx_xml_parse_err(c, *c->p, __FUNCTION__, "Did not find expected quote char (single or double), saw");
+    return 0;
+  }
+  quote = *c->p;
+  ++c->p;
+  data = c->p;	
+  
+  ZX_LOOK_FOR(c, quote);
+  return data;
+ look_for_not_found:
+  zx_xml_parse_err(c, quote, __FUNCTION__, "char not found");
+  return 0;
+}
+
+void zx_xml_parse_err(struct zx_ctx* c, char quote, const char* func, const char* msg) {
+  char* errloc = MAX(c->p - 20, c->base);
+  ERR("%s: %s: char(%c) pos=%d (%.*s)", func, msg, quote,
+      c->p - c->base, MIN(c->lim - errloc, 40), errloc);
+}
+
+/* Add inclusive namespaces. */
+
+int zx_len_inc_ns(struct zx_ctx* c, struct zx_ns_s** pop_seenp)
+{
+  int len = 0;
+  struct zx_ns_s* ns;
+  for (ns = c->inc_ns; ns; ns = ns->inc_n)
+    len += zx_len_xmlns_if_not_seen(c, ns, pop_seenp);
+  c->inc_ns_len = 0;
+  return len;
+}
+
+void zx_add_inc_ns(struct zx_ctx* c, struct zx_ns_s** pop_seenp)
+{
+  struct zx_ns_s* ns;
+  for (ns = c->inc_ns; ns; ns = ns->inc_n)
+    zx_add_xmlns_if_not_seen(c, ns, pop_seenp);
+  c->inc_ns = 0;
+}
+
+char* zx_enc_inc_ns(struct zx_ctx* c, char* p, struct zx_ns_s** pop_seenp)
+{
+  struct zx_ns_s* ns;
+  for (ns = c->inc_ns; ns; ns = ns->inc_n)
+    p = zx_enc_xmlns_if_not_seen(c, p, ns, pop_seenp);
+  c->inc_ns = 0;
+  return p;
+}
+
 /* EOF -- zxlib.c */
